@@ -5,17 +5,16 @@
 //
 package org.dogtagpki.acme.server;
 
-import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.FileReader;
 import java.math.BigInteger;
+import java.net.URL;
 import java.security.KeyFactory;
 import java.security.MessageDigest;
 import java.security.Principal;
 import java.security.PublicKey;
 import java.security.SecureRandom;
 import java.security.Signature;
-import java.security.cert.CertificateFactory;
 import java.security.cert.X509Certificate;
 import java.security.spec.RSAPublicKeySpec;
 import java.util.Collection;
@@ -28,6 +27,7 @@ import java.util.Map;
 import java.util.Properties;
 import java.util.Random;
 import java.util.Set;
+import java.util.concurrent.ConcurrentHashMap;
 import java.util.function.Consumer;
 
 import javax.servlet.ServletContextEvent;
@@ -70,6 +70,7 @@ import org.mozilla.jss.netscape.security.util.ObjectIdentifier;
 import org.mozilla.jss.netscape.security.util.Utils;
 import org.mozilla.jss.netscape.security.x509.CertAttrSet;
 import org.mozilla.jss.netscape.security.x509.DNSName;
+import org.mozilla.jss.netscape.security.x509.Extension;
 import org.mozilla.jss.netscape.security.x509.Extensions;
 import org.mozilla.jss.netscape.security.x509.GeneralName;
 import org.mozilla.jss.netscape.security.x509.GeneralNameInterface;
@@ -117,6 +118,9 @@ public class ACMEEngine implements ServletContextListener {
     private ACMERealmConfig realmConfig;
     private ACMERealm realm;
 
+    private boolean noncesPersistent;
+    private Map<String, ACMENonce> nonces = new ConcurrentHashMap<>();
+
     public static ACMEEngine getInstance() {
         return INSTANCE;
     }
@@ -203,6 +207,10 @@ public class ACMEEngine implements ServletContextListener {
         config.setEnabled(enabled);
     }
 
+    public URL getBaseURL() {
+        return config.getBaseURL();
+    }
+
     public void loadConfig(String filename) throws Exception {
 
         File configFile = new File(filename);
@@ -219,6 +227,8 @@ public class ACMEEngine implements ServletContextListener {
 
         config = ACMEEngineConfig.fromProperties(props);
         logger.info("- enabled: " + config.isEnabled());
+        logger.info("- base URL: " + config.getBaseURL());
+        logger.info("- nonces persistent: " + config.getNoncesPersistent());
 
         ACMEPolicyConfig policyConfig = config.getPolicyConfig();
         logger.info("- wildcard: " + policyConfig.getEnableWildcards());
@@ -280,7 +290,7 @@ public class ACMEEngine implements ServletContextListener {
         String className = databaseConfig.getClassName();
         Class<ACMEDatabase> databaseClass = (Class<ACMEDatabase>) Class.forName(className);
 
-        database = databaseClass.newInstance();
+        database = databaseClass.getDeclaredConstructor().newInstance();
         database.setConfig(databaseConfig);
         database.init();
     }
@@ -311,7 +321,7 @@ public class ACMEEngine implements ServletContextListener {
             String className = validatorConfig.getClassName();
             Class<ACMEValidator> validatorClass = (Class<ACMEValidator>) Class.forName(className);
 
-            ACMEValidator validator = validatorClass.newInstance();
+            ACMEValidator validator = validatorClass.getDeclaredConstructor().newInstance();
             validator.setConfig(validatorConfig);
             validator.init();
 
@@ -341,7 +351,7 @@ public class ACMEEngine implements ServletContextListener {
         String className = issuerConfig.getClassName();
         Class<ACMEIssuer> issuerClass = (Class<ACMEIssuer>) Class.forName(className);
 
-        issuer = issuerClass.newInstance();
+        issuer = issuerClass.getDeclaredConstructor().newInstance();
         issuer.setConfig(issuerConfig);
         issuer.init();
     }
@@ -392,7 +402,7 @@ public class ACMEEngine implements ServletContextListener {
             configSourceClass =
                 (Class<ACMEEngineConfigSource>) Class.forName(className);
         }
-        engineConfigSource = configSourceClass.newInstance();
+        engineConfigSource = configSourceClass.getDeclaredConstructor().newInstance();
 
         // We pass to the ConfigSource only the callbacks needed to set
         // the configuration (Consumer<T>).  This abstraction ensures the
@@ -445,7 +455,7 @@ public class ACMEEngine implements ServletContextListener {
         String className = realmConfig.getClassName();
         Class<ACMERealm> realmClass = (Class<ACMERealm>) Class.forName(className);
 
-        realm = realmClass.newInstance();
+        realm = realmClass.getDeclaredConstructor().newInstance();
         realm.setConfig(realmConfig);
         realm.init();
 
@@ -493,6 +503,9 @@ public class ACMEEngine implements ServletContextListener {
         logger.info("ACME configuration directory: " + acmeConfDir);
         loadConfig(acmeConfDir + File.separator + "engine.conf");
 
+        Boolean noncePersistent = config.getNoncesPersistent();
+        this.noncesPersistent =  noncePersistent != null ? noncePersistent : false;
+
         initRandomGenerator();
         initMetadata(acmeConfDir + File.separator + "metadata.conf");
         initDatabase(acmeConfDir + File.separator + "database.conf");
@@ -588,7 +601,12 @@ public class ACMEEngine implements ServletContextListener {
         Date expirationTime = policy.getNonceExpirationTime(currentTime);
         nonce.setExpirationTime(expirationTime);
 
-        database.addNonce(nonce);
+        if (noncesPersistent) {
+            database.addNonce(nonce);
+        } else {
+            nonces.put(nonce.getID(), nonce);
+        }
+
         logger.info("Created nonce: " + nonce);
 
         return nonce;
@@ -596,7 +614,13 @@ public class ACMEEngine implements ServletContextListener {
 
     public void validateNonce(String value) throws Exception {
 
-        ACMENonce nonce = database.removeNonce(value);
+        ACMENonce nonce;
+
+        if (noncesPersistent) {
+            nonce = database.removeNonce(value);
+        } else {
+            nonce = nonces.remove(value);
+        }
 
         if (nonce == null) {
             // TODO: generate proper exception
@@ -614,6 +638,19 @@ public class ACMEEngine implements ServletContextListener {
         logger.info("Valid nonce: " + value);
     }
 
+    public void removeExpiredRecords(Date currentTime) throws Exception {
+
+        if (noncesPersistent) {
+            database.removeExpiredNonces(currentTime);
+        } else {
+            nonces.values().removeIf(n -> !currentTime.before(n.getExpirationTime()));
+        }
+
+        database.removeExpiredAuthorizations(currentTime);
+        database.removeExpiredOrders(currentTime);
+        database.removeExpiredCertificates(currentTime);
+    }
+
     public void validateJWS(JWS jws, String alg, JWK jwk) throws Exception {
 
         // TODO: support other algorithms
@@ -638,7 +675,16 @@ public class ACMEEngine implements ServletContextListener {
             publicKey = keyFactory.generatePublic(keySpec);
 
         } else {
-            throw new Exception("Unsupported JWS algorithm: " + alg);
+            ResponseBuilder builder = Response.status(Response.Status.BAD_REQUEST);
+            builder.type("application/problem+json");
+
+            ACMEError error = new ACMEError();
+            error.setType("urn:ietf:params:acme:error:badSignatureAlgorithm");
+            error.setDetail("Signature of type " + alg + " not supported\n" +
+                    "Try again with RS256.");
+            builder.entity(error);
+
+            throw new WebApplicationException(builder.build());
         }
 
         validateJWS(jws, signer, publicKey);
@@ -955,13 +1001,14 @@ public class ACMEEngine implements ServletContextListener {
         while (extNames.hasMoreElements()) {
 
             String name = extNames.nextElement();
-            Object value = extensions.get(name);
+            Extension extension = (Extension) extensions.get(name);
             logger.info("  - " + name);
+            logger.info("    - critical: " + extension.isCritical());
 
             // TODO: support other extensions
 
-            if (value instanceof SubjectAlternativeNameExtension) {
-                SubjectAlternativeNameExtension sanExt = (SubjectAlternativeNameExtension) value;
+            if (extension instanceof SubjectAlternativeNameExtension) {
+                SubjectAlternativeNameExtension sanExt = (SubjectAlternativeNameExtension) extension;
                 parseCSRSAN(sanExt, dnsNames);
             }
         }
@@ -1012,12 +1059,7 @@ public class ACMEEngine implements ServletContextListener {
         Date now = new Date();
         String certBase64 = revocation.getCertificate();
         byte[] certData = Utils.base64decode(certBase64);
-
-        X509Certificate cert;
-        try (ByteArrayInputStream is = new ByteArrayInputStream(certData)) {
-            CertificateFactory cf = CertificateFactory.getInstance("X.509");
-            cert = (X509Certificate) cf.generateCertificate(is);
-        }
+        X509CertImpl cert = new X509CertImpl(certData);
 
         String certID = issuer.getCertificateID(cert);
 
@@ -1152,6 +1194,7 @@ public class ACMEEngine implements ServletContextListener {
         return identifiers;
     }
 
+    @Override
     public void contextInitialized(ServletContextEvent event) {
 
         String path = event.getServletContext().getContextPath();
@@ -1170,6 +1213,7 @@ public class ACMEEngine implements ServletContextListener {
         }
     }
 
+    @Override
     public void contextDestroyed(ServletContextEvent event) {
 
         try {
