@@ -452,6 +452,8 @@ class NSSDatabase(object):
 
     def add_cert(self, nickname, cert_file, token=None, trust_attributes=None):
 
+        logger.debug('nssdb.add_cert')
+
         tmpdir = tempfile.mkdtemp()
         try:
             token = self.get_effective_token(token)
@@ -536,8 +538,8 @@ class NSSDatabase(object):
             cmd.extend(['-C', self.internal_password_file])
 
         cmd.extend([
-            'nss-cert-import',
-            '--cert', cert_file
+            'client-cert-import',
+            '--ca-cert', cert_file
         ])
 
         if trust_attributes:
@@ -815,6 +817,51 @@ class NSSDatabase(object):
         finally:
             shutil.rmtree(tmpdir)
 
+    def create_request_with_wrap_key(
+            self,
+            subject_dn,
+            request_file,
+            token=None,
+            key_size=None):
+
+        """
+        Generate a CSR for certs that need to perform key wrapping/unwrapping
+        on an HSM (dependent on make/model). Only RSA key type is supported.
+        """
+
+        tmpdir = tempfile.mkdtemp()
+        try:
+
+            cmd = [
+                'PKCS10Client',
+                '-d', self.directory
+            ]
+
+            token = self.get_effective_token(token)
+            password_file = self.get_password_file(tmpdir, token)
+
+            if token:
+                cmd.extend(['-h', token])
+
+            if password_file:
+                cmd.extend(['-P', password_file])
+
+            size = key_size if key_size else 2048
+            cmd.extend([
+                '-a', 'rsa',
+                '-l', str(size),
+                '-n', subject_dn,
+                '-w',
+                '-v',
+                '-o', request_file,
+            ])
+
+            logger.debug('Command: %s', ' '.join(map(str, cmd)))
+            subprocess.check_call(cmd)
+
+        finally:
+            shutil.rmtree(tmpdir)
+
     def __generate_key(
             self, tmpdir,
             key_type=None, key_size=None, curve=None,
@@ -1158,6 +1205,9 @@ class NSSDatabase(object):
         :return: Trust flag
         :rtype: str
         """
+        logger.debug('get_trust')
+        cert_trust = None
+
         tmpdir = tempfile.mkdtemp()
         try:
             token = self.get_effective_token(token)
@@ -1173,6 +1223,8 @@ class NSSDatabase(object):
                 cmd.extend(['-h', token])
                 fullname = token + ':' + fullname
 
+            logger.debug('fullname = %s', fullname)
+
             if password_file:
                 cmd.extend(['-f', password_file])
 
@@ -1187,6 +1239,7 @@ class NSSDatabase(object):
             if error:
                 # certutil returned an error
                 # raise exception unless its not cert not found
+                logger.error('error : %s', error)
                 if error.startswith(b'certutil: Could not find cert: '):
                     return None
 
@@ -1254,6 +1307,8 @@ class NSSDatabase(object):
     def get_cert(self, nickname, token=None, output_format='pem',
                  output_text=False):
 
+        logger.info('nssdb.get_cert')
+
         if output_format == 'pem':
             output_format_option = '-a'
 
@@ -1309,16 +1364,22 @@ class NSSDatabase(object):
 
             if not cert_data:
                 # certutil did not return data
+                logger.info('certutil did not return data')
                 return None
+            else:
+                logger.info('certutil returned data for cert')
 
             if p.returncode != 0:
                 logger.warning('certutil returned non-zero exit code (bug #1539996)')
+                logger.info('certutil returned errocode: %s', p.returncode)
 
             if output_format == 'base64':
                 cert_data = base64.b64encode(cert_data).decode('utf-8')
             if output_text and not isinstance(cert_data, six.string_types):
                 cert_data = cert_data.decode('ascii')
 
+            logger.info('nssdb.get_cert ends')
+
             return cert_data
 
         finally:
@@ -1326,6 +1387,7 @@ class NSSDatabase(object):
 
     def get_cert_info(self, nickname, token=None):
 
+        logger.debug('nssdb.get_cert_info begins: %s', nickname)
         cert_pem = self.get_cert(nickname=nickname, token=token)
 
         if not cert_pem:
@@ -1346,6 +1408,8 @@ class NSSDatabase(object):
         cert['not_after'] = self.convert_time_to_millis(cert_obj.not_valid_after)
         cert['trust_flags'] = self.get_trust(nickname=nickname, token=token)
 
+        logger.debug('nssdb.get_cert_info ends: %s', nickname)
+
         return cert
 
     @staticmethod
@@ -1509,10 +1573,13 @@ class NSSDatabase(object):
 
         tmpdir = tempfile.mkdtemp()
 
+        logger.debug('nssdb.import_cert_chain begins')
+
         try:
             file_type = get_file_type(cert_chain_file)
 
             if file_type == 'cert':  # import single PEM cert
+                logger.debug('type is cert')
                 self.add_cert(
                     nickname=nickname,
                     cert_file=cert_chain_file,
@@ -1527,6 +1594,7 @@ class NSSDatabase(object):
                 )
 
             elif file_type == 'pkcs7':  # import PKCS #7 cert chain
+                logger.debug('type is pkcs7')
                 self.import_pkcs7(
                     pkcs7_file=cert_chain_file,
                     nickname=nickname,
@@ -1541,6 +1609,7 @@ class NSSDatabase(object):
                 return base64_data, [nickname]
 
             else:  # import PKCS #7 data without header/footer
+                logger.debug('type is pkcs7 without header/footer')
                 with open(cert_chain_file, 'r') as f:
                     base64_data = f.read()
 
@@ -1566,6 +1635,7 @@ class NSSDatabase(object):
 
         finally:
             shutil.rmtree(tmpdir)
+            logger.info('import_cert_chain ends')
 
     def import_pkcs7(
             self,
@@ -1575,6 +1645,8 @@ class NSSDatabase(object):
             token=None,
             trust_attributes=None):
 
+        logger.debug('import_pkcs7')
+
         if not nickname:
 
             # Import certificate chain without nickname
@@ -1616,7 +1688,7 @@ class NSSDatabase(object):
                 'pki',
                 '-d', self.directory,
                 'pkcs7-cert-export',
-                '--pkcs7-file', pkcs7_file,
+                '--pkcs7', pkcs7_file,
                 '--output-prefix', prefix,
                 '--output-suffix', suffix
             ]
@@ -1632,6 +1704,7 @@ class NSSDatabase(object):
                     break
                 n = n + 1
 
+            logger.debug('Number of certs in pkcs#7 to import: %s', n)
             # Import CA certs with default nicknames and trust attributes.
             for i in range(0, n - 1):
                 cert_file = prefix + str(i) + suffix
