@@ -308,6 +308,7 @@ class CertificateSetupRequest(object):
     def __init__(self):
         self.installToken = None
         self.clone = 'false'
+        self.systemCert = None
 
 
 class CertificateSetupResponse(object):
@@ -325,11 +326,6 @@ class AdminSetupResponse(object):
         pass
 
 
-class FinalizeConfigRequest(object):
-    def __init__(self):
-        self.installToken = None
-
-
 class SystemCertData(object):
     """
     Class used to represent the data for a system certificate, which is
@@ -357,22 +353,37 @@ class SystemConfigClient(object):
 
         self.connection = connection
 
+        self.load_cert_url = '/rest/installer/loadCert'
         self.setup_cert_url = '/rest/installer/setupCert'
         self.setup_admin_url = '/rest/installer/setupAdmin'
         self.backup_keys_url = '/rest/installer/backupKeys'
         self.setup_security_domain_url = '/rest/installer/setupSecurityDomain'
-        self.finalize_config_url = '/rest/installer/finalizeConfiguration'
 
         if connection.subsystem is None:
 
             if subsystem is None:
                 raise Exception('Missing subsystem for SystemConfigClient')
 
+            self.load_cert_url = '/' + subsystem + self.load_cert_url
             self.setup_cert_url = '/' + subsystem + self.setup_cert_url
             self.setup_admin_url = '/' + subsystem + self.setup_admin_url
             self.backup_keys_url = '/' + subsystem + self.backup_keys_url
             self.setup_security_domain_url = '/' + subsystem + self.setup_security_domain_url
-            self.finalize_config_url = '/' + subsystem + self.finalize_config_url
+
+    def loadCert(self, request):
+        """
+        Load existing certificate.
+
+        :param request: Certificate setup request
+        :type request: CertificateSetupRequest
+        """
+        data = json.dumps(request, cls=pki.encoder.CustomTypeEncoder)
+        headers = {'Content-type': 'application/json'}
+
+        self.connection.post(
+            self.load_cert_url,
+            data,
+            headers)
 
     def setupCert(self, request):
         """
@@ -413,21 +424,6 @@ class SystemConfigClient(object):
             headers)
         return response.json()
 
-    def finalizeConfiguration(self, request):
-        """
-        Finalize server configuration.
-
-        :param request: Finalize configuration request
-        :type request: FinalizeConfigRequest
-        """
-        data = json.dumps(request, cls=pki.encoder.CustomTypeEncoder)
-        headers = {'Content-type': 'application/json',
-                   'Accept': 'application/json'}
-        self.connection.post(
-            self.finalize_config_url,
-            data,
-            headers)
-
 
 class SystemStatusClient(object):
     """
@@ -470,5 +466,4 @@ pki.encoder.NOTYPES['CertificateSetupRequest'] = CertificateSetupRequest
 pki.encoder.NOTYPES['CertificateSetupResponse'] = CertificateSetupResponse
 pki.encoder.NOTYPES['AdminSetupRequest'] = AdminSetupRequest
 pki.encoder.NOTYPES['AdminSetupResponse'] = AdminSetupResponse
-pki.encoder.NOTYPES['FinalizeConfigRequest'] = FinalizeConfigRequest
 pki.encoder.NOTYPES['SystemCertData'] = SystemCertData
