@@ -20,12 +20,11 @@
 #
 
 from __future__ import absolute_import
-from __future__ import print_function
 import functools
 import logging
 import os
+import pathlib
 import re
-import shutil
 
 import pki
 import pki.util
@@ -57,6 +56,9 @@ class PKIUpgradeTracker(object):
 
         self.properties = pki.PropertyFile(filename, delimiter)
 
+        # run all scriptlets for each upgrade version
+        self.remove_index()
+
     def remove(self):
 
         logger.info('Removing %s tracker', self.name)
@@ -73,16 +75,14 @@ class PKIUpgradeTracker(object):
 
     def show(self):
 
-        print(self.name + ':')
+        print('%s:' % self.name)
 
         version = self.get_version()
-        print('  Configuration version: ' + str(version))
+        print('  Configuration version: %s' % version)
 
         index = self.get_index()
         if index > 0:
-            print('  Last completed scriptlet: ' + str(index))
-
-        print()
+            print('  Last completed scriptlet: %s' % index)
 
     def get_index(self):
 
@@ -189,180 +189,12 @@ class PKIUpgradeScriptlet(object):
     def get_backup_dir(self):
         return BACKUP_DIR + '/' + str(self.version) + '/' + str(self.index)
 
-    def can_upgrade(self):
-
-        # A scriptlet can run if the version matches the tracker and
-        # the index is the next to be executed.
-
-        tracker = self.upgrader.get_tracker()
-
-        return self.version == tracker.get_version() and \
-            self.index == tracker.get_index() + 1
-
-    def update_tracker(self):
-
-        # Increment the index in the tracker. If it's the last scriptlet
-        # in this version, update the tracker version.
-
-        tracker = self.upgrader.get_tracker()
-        self.backup(tracker.filename)
-
-        if not self.last:
-            tracker.set_index(self.index)
-
-        else:
-            tracker.remove_index()
-            tracker.set_version(self.version.next)
-
     def upgrade_system(self):
         # Callback method to upgrade the system.
         pass
 
-    def init(self):
-
-        backup_dir = self.get_backup_dir()
-
-        if os.path.exists(backup_dir):
-            # remove old backup dir
-            shutil.rmtree(backup_dir)
-
-        # create backup dir
-        os.makedirs(backup_dir)
-
-    def upgrade(self):
-
-        try:
-            if not self.can_upgrade():
-                logger.info('Skipping system')
-                return
-
-            logger.info('Upgrading system')
-            self.upgrade_system()
-            self.update_tracker()
-
-        except Exception as e:
-
-            if logger.isEnabledFor(logging.INFO):
-                logger.exception(e)
-            else:
-                logger.error(e)
-
-            message = 'Failed upgrading system.'
-            if self.upgrader.silent:
-                print(message)
-            else:
-                result = pki.util.read_text(
-                    message + ' Continue (Yes/No)',
-                    options=['Y', 'N'], default='Y', delimiter='?',
-                    case_sensitive=False).lower()
-                if result == 'y':
-                    return
-
-            raise pki.PKIException('Upgrade failed: %s' % e, e)
-
-    def revert(self):
-
-        backup_dir = self.get_backup_dir()
-
-        if not os.path.exists(backup_dir):
-            return
-
-        oldfiles = backup_dir + '/oldfiles'
-        if os.path.exists(oldfiles):
-
-            # restore all backed up files
-            for sourcepath, _, filenames in os.walk(oldfiles):
-                # unused item _ for dirnames
-
-                destpath = sourcepath[len(oldfiles):]
-                if destpath == '':
-                    destpath = '/'
-
-                if not os.path.isdir(destpath):
-                    logger.info('Restoring %s', destpath)
-                    pki.util.copydirs(sourcepath, destpath, force=True)
-
-                for filename in filenames:
-                    sourcefile = os.path.join(sourcepath, filename)
-                    targetfile = os.path.join(destpath, filename)
-
-                    logger.info('Restoring %s', targetfile)
-                    pki.util.copyfile(sourcefile, targetfile, force=True)
-
-        newfiles = backup_dir + '/newfiles'
-        if os.path.exists(newfiles):
-
-            # get paths that did not exist before upgrade
-            paths = []
-            with open(newfiles, 'r') as f:
-                for path in f:
-                    path = path.strip('\n')
-                    paths.append(path)
-
-            # remove paths in reverse order
-            paths.reverse()
-            for path in paths:
-
-                if not os.path.exists(path):
-                    continue
-                logger.info('Deleting %s', path)
-
-                if os.path.isfile(path):
-                    os.remove(path)
-                else:
-                    shutil.rmtree(path)
-
     def backup(self, path):
-
-        backup_dir = self.get_backup_dir()
-
-        if not os.path.exists(backup_dir):
-            os.makedirs(backup_dir)
-
-        if os.path.exists(path):
-
-            # if path exists, keep a copy
-
-            oldfiles = backup_dir + '/oldfiles'
-            if not os.path.exists(oldfiles):
-                os.mkdir(oldfiles)
-
-            dest = oldfiles + path
-
-            sourceparent = os.path.dirname(path)
-            destparent = os.path.dirname(dest)
-
-            pki.util.copydirs(sourceparent, destparent, force=True)
-
-            if os.path.isfile(path):
-                logger.info('Saving %s', path)
-                # do not overwrite initial backup
-                pki.util.copyfile(path, dest, force=False)
-
-            else:
-                for sourcepath, _, filenames in os.walk(path):
-
-                    relpath = sourcepath[len(path):]
-                    destpath = dest + relpath
-
-                    logger.info('Saving %s', sourcepath)
-                    pki.util.copydirs(sourcepath, destpath, force=True)
-
-                    for filename in filenames:
-                        sourcefile = os.path.join(sourcepath, filename)
-                        targetfile = os.path.join(destpath, filename)
-
-                        logger.info('Saving %s', sourcefile)
-                        # do not overwrite initial backup
-                        pki.util.copyfile(sourcefile, targetfile, force=False)
-
-        else:
-
-            # otherwise, record the name
-
-            logger.info('Recording %s', path)
-            with open(backup_dir + '/newfiles', 'a') as f:
-                f.write(path + '\n')
+        self.upgrader.backup(self, path)
 
     def __eq__(self, other):
         return self.version == other.version and self.index == other.index
@@ -379,20 +211,10 @@ class PKIUpgradeScriptlet(object):
 
 class PKIUpgrader(object):
 
-    def __init__(self, upgrade_dir=UPGRADE_DIR, version=None, index=None,
-                 silent=False):
+    def __init__(self, upgrade_dir=UPGRADE_DIR):
 
         self.upgrade_dir = upgrade_dir
-        self.version = version
-        self.index = index
-
-        self.silent = silent
-
-        if version and not os.path.exists(self.version_dir(version)):
-            raise pki.PKIException(
-                'Invalid scriptlet version: ' + str(version))
-
-        self.system_tracker = None
+        self.tracker = None
 
     def version_dir(self, version):
 
@@ -440,9 +262,9 @@ class PKIUpgrader(object):
         if not upgrade_path or upgrade_path[-1] != target_version:
             upgrade_path.append(target_version)
 
-        logging.debug('Upgrade path:')
+        logger.debug('Upgrade path:')
         for version in upgrade_path:
-            logging.debug(' - %s', version)
+            logger.debug(' - %s', version)
 
         versions = []
 
@@ -454,11 +276,7 @@ class PKIUpgrader(object):
             else:
                 version.next = target_version
 
-            # if no scriptlet version is specified, add all versions to the list
-            # if scriptlet version is specified, add only that version to the
-            # list
-            if not self.version or str(version) == self.version:
-                versions.append(version)
+            versions.append(version)
 
         return versions
 
@@ -481,9 +299,6 @@ class PKIUpgrader(object):
             index = int(match.group(1))
             classname = match.group(2)
 
-            if self.index and index != self.index:
-                continue
-
             # load scriptlet class
             variables = {}
             absname = os.path.join(version_dir, filename)
@@ -494,45 +309,48 @@ class PKIUpgrader(object):
             # create scriptlet object
             scriptlet = variables[classname]()
 
-            scriptlet.upgrader = self
             scriptlet.version = version
             scriptlet.index = index
-            scriptlet.last = index == len(filenames)
 
             scriptlets.append(scriptlet)
 
         # sort scriptlets based on index
         scriptlets.sort()
 
+        if scriptlets:
+            scriptlets[-1].last = True
+
         return scriptlets
 
     def get_tracker(self):
 
-        if self.system_tracker:
-            tracker = self.system_tracker
+        if self.tracker:
+            return self.tracker
 
-        else:
-            tracker = PKIUpgradeTracker(
-                'system',
-                SYSTEM_TRACKER,
-                delimiter=': ',
-                version_key='Configuration-Version',
-                index_key='Scriptlet-Index')
-            self.system_tracker = tracker
+        self.tracker = PKIUpgradeTracker(
+            'system',
+            SYSTEM_TRACKER,
+            delimiter=': ',
+            version_key='Configuration-Version',
+            index_key='Scriptlet-Index')
 
-        return tracker
+        return self.tracker
 
     def get_current_version(self):
 
         current_version = self.get_tracker().get_version()
-        logging.debug('Current version: %s', current_version)
+
+        if not current_version:
+            current_version = self.get_target_version()
+
+        logger.debug('Current version: %s', current_version)
 
         return current_version
 
     def get_target_version(self):
 
         target_version = pki.util.Version(pki.specification_version())
-        logging.debug('Target version: %s', target_version)
+        logger.debug('Target version: %s', target_version)
 
         return target_version
 
@@ -544,71 +362,119 @@ class PKIUpgrader(object):
         return current_version == target_version
 
     def validate(self):
-
         if not self.is_complete():
-            log_file = '/var/log/pki/pki-upgrade-%s.log' % self.get_target_version()
-            raise Exception('Upgrade incomplete: see %s' % log_file)
+            raise Exception('Incomplete upgrade')
+
+    def touch(self, path):
+        pathlib.Path(path).touch()
+
+    def makedirs(self, path, exist_ok=False):
+        os.makedirs(path, exist_ok=exist_ok)
+
+    def copydirs(self, source, dest, force=False):
+        pki.util.copydirs(source, dest, force=force)
+
+    def copyfile(self, source, dest, force=False):
+        pki.util.copyfile(source, dest, force=force)
+
+    def record(self, scriptlet, path):
+
+        backup_dir = scriptlet.get_backup_dir()
+        filename = backup_dir + '/newfiles'
+
+        self.touch(filename)
+        with open(filename, 'a') as f:
+            f.write(path + '\n')
+
+    def backup(self, scriptlet, path):
+
+        backup_dir = scriptlet.get_backup_dir()
+
+        self.makedirs(backup_dir, exist_ok=True)
+
+        if not os.path.exists(path):
+
+            # if path does not exists, record the name
+            logger.info('Recording %s', path)
+            self.record(scriptlet, path)
+            return
+
+        # otherwise, keep a copy
+
+        oldfiles = backup_dir + '/oldfiles'
+        self.makedirs(oldfiles, exist_ok=True)
+
+        dest = oldfiles + path
+
+        sourceparent = os.path.dirname(path)
+        destparent = os.path.dirname(dest)
+
+        if not os.path.exists(destparent):
+            self.copydirs(sourceparent, destparent, force=True)
+
+        if os.path.isfile(path):
+
+            # backup file
+            if not os.path.exists(dest):
+                logger.info('Saving %s', path)
+                self.copyfile(path, dest)
+
+            return
+
+        # backup folder
+
+        for sourcepath, _, filenames in os.walk(path):
+
+            relpath = sourcepath[len(path):]
+            destpath = dest + relpath
+
+            if not os.path.exists(destpath):
+                logger.info('Saving %s', sourcepath)
+                self.copydirs(sourcepath, destpath, force=True)
+
+            for filename in filenames:
+                sourcefile = os.path.join(sourcepath, filename)
+                targetfile = os.path.join(destpath, filename)
+
+                if not os.path.exists(targetfile):
+                    logger.info('Saving %s', sourcefile)
+                    self.copyfile(sourcefile, targetfile)
 
     def upgrade_version(self, version):
 
-        if version == version.next:
-            print('Upgrading version %s:' % version)
-        else:
-            print('Upgrading from version %s to %s:' % (version, version.next))
-
         scriptlets = self.scriptlets(version)
 
         if len(scriptlets) == 0:
 
-            print('No upgrade scriptlets.')
-
             self.set_tracker(version.next)
             return
 
         # execute scriptlets
         for scriptlet in scriptlets:
 
-            message = str(scriptlet.index) + '. ' + scriptlet.message
+            logger.info('Running upgrade script %s-%s: %s',
+                        version, scriptlet.index, scriptlet.message)
 
-            if self.silent:
-                print(message)
+            self.init_scriptlet(scriptlet)
+            self.run_scriptlet(scriptlet)
+            self.update_tracker(scriptlet)
 
-            else:
-                result = pki.util.read_text(
-                    message + ' (Yes/No)',
-                    options=['Y', 'N'],
-                    default='Y',
-                    case_sensitive=False).lower()
+    def init_scriptlet(self, scriptlet):
 
-                if result == 'n':
-                    raise pki.PKIException('Upgrade canceled.')
+        scriptlet.upgrader = self
+        backup_dir = scriptlet.get_backup_dir()
 
-            try:
-                scriptlet.init()
-                scriptlet.upgrade()
+        if os.path.exists(backup_dir):
+            logger.debug('Command: rm -rf %s', backup_dir)
+            pki.util.rmtree(backup_dir)
 
-            except Exception as e:  # pylint: disable=W0703
+        logger.debug('Command: mkdir -p %s', backup_dir)
+        self.makedirs(backup_dir)
 
-                print()
+    def run_scriptlet(self, scriptlet):
 
-                message = 'Upgrade failed: %s' % e
-
-                if logger.isEnabledFor(logging.INFO):
-                    logger.exception(e)
-                else:
-                    logger.error(e)
-
-                print()
-
-                result = pki.util.read_text(
-                    'Continue (Yes/No)',
-                    options=['Y', 'N'],
-                    default='Y',
-                    delimiter='?',
-                    case_sensitive=False).lower()
-
-                if result == 'n':
-                    raise pki.PKIException(message, e)
+        logger.info('Upgrading system')
+        scriptlet.upgrade_system()
 
     def upgrade(self):
 
@@ -616,60 +482,70 @@ class PKIUpgrader(object):
 
         for version in versions:
             self.upgrade_version(version)
-            print()
 
-        if self.is_complete():
-            print('Upgrade complete.')
+    def revert_scriptlet(self, scriptlet):
 
-        else:
-            self.show_tracker()
-            print('Upgrade incomplete.')
+        backup_dir = scriptlet.get_backup_dir()
+
+        if not os.path.exists(backup_dir):
+            return
+
+        oldfiles = backup_dir + '/oldfiles'
+        if os.path.exists(oldfiles):
+
+            # restore all backed up files
+            for sourcepath, _, filenames in os.walk(oldfiles):
+                # unused item _ for dirnames
+
+                destpath = sourcepath[len(oldfiles):]
+                if destpath == '':
+                    destpath = '/'
+
+                if not os.path.isdir(destpath):
+                    logger.info('Restoring %s', destpath)
+                    self.copydirs(sourcepath, destpath, force=True)
+
+                for filename in filenames:
+                    sourcefile = os.path.join(sourcepath, filename)
+                    targetfile = os.path.join(destpath, filename)
+
+                    logger.info('Restoring %s', targetfile)
+                    self.copyfile(sourcefile, targetfile, force=True)
+
+        newfiles = backup_dir + '/newfiles'
+        if os.path.exists(newfiles):
+
+            # get paths that did not exist before upgrade
+            paths = []
+
+            with open(newfiles, 'r') as f:
+                for path in f:
+                    path = path.strip('\n')
+                    paths.append(path)
+
+            # remove paths in reverse order
+            paths.reverse()
+
+            for path in paths:
+
+                if not os.path.exists(path):
+                    continue
+
+                logger.info('Deleting %s', path)
+                if os.path.isfile(path):
+                    os.remove(path)
+                else:
+                    pki.util.rmtree(path)
 
     def revert_version(self, version):
 
-        print('Reverting to version ' + str(version) + ':')
-
         scriptlets = self.scriptlets(version)
         scriptlets.reverse()
 
         for scriptlet in scriptlets:
 
-            message = str(scriptlet.index) + '. ' + scriptlet.message
-
-            if self.silent:
-                print(message)
-
-            else:
-                result = pki.util.read_text(
-                    message + ' (Yes/No)',
-                    options=['Y', 'N'], default='Y',
-                    case_sensitive=False).lower()
-
-                if result == 'n':
-                    raise pki.PKIException('Revert canceled.')
-
-            try:
-                scriptlet.revert()
-
-            except Exception as e:  # pylint: disable=W0703
-
-                print()
-
-                message = 'Revert failed: %s' % e
-
-                if logger.isEnabledFor(logging.INFO):
-                    logger.exception(e)
-                else:
-                    logger.error(e)
-
-                print()
-
-                result = pki.util.read_text(
-                    'Continue (Yes/No)', options=['Y', 'N'],
-                    default='Y', delimiter='?', case_sensitive=False).lower()
-
-                if result == 'n':
-                    raise pki.PKIException(message, e)
+            logger.info('Reverting %s: %s. %s', version, scriptlet.index, scriptlet.message)
+            self.revert_scriptlet(scriptlet)
 
         self.set_tracker(version)
 
@@ -689,7 +565,7 @@ class PKIUpgrader(object):
             self.revert_version(version)
             return
 
-        print('Unable to revert from version ' + str(current_version) + '.')
+        logger.info('Unable to revert from version %s.', current_version)
 
     def show_tracker(self):
 
@@ -700,18 +576,25 @@ class PKIUpgrader(object):
 
         self.show_tracker()
 
-        if self.is_complete():
-            print('Upgrade complete.')
-
-        else:
-            print('Upgrade incomplete.')
-
     def set_tracker(self, version):
 
         tracker = self.get_tracker()
         tracker.set(version)
 
-        print('Tracker has been set to version ' + str(version) + '.')
+    def update_tracker(self, scriptlet):
+
+        # Increment the index in the tracker. If it's the last scriptlet
+        # in this version, update the tracker version.
+
+        tracker = self.get_tracker()
+        scriptlet.backup(tracker.filename)
+
+        if not scriptlet.last:
+            tracker.set_index(scriptlet.index)
+
+        else:
+            tracker.remove_index()
+            tracker.set_version(scriptlet.version.next)
 
     def reset_tracker(self):
 
@@ -722,5 +605,3 @@ class PKIUpgrader(object):
 
         tracker = self.get_tracker()
         tracker.remove()
-
-        print('Tracker has been removed.')
