@@ -17,20 +17,21 @@
 // --- END COPYRIGHT BLOCK ---
 package com.netscape.certsrv.system;
 
-import javax.xml.bind.annotation.XmlAccessType;
-import javax.xml.bind.annotation.XmlAccessorType;
-import javax.xml.bind.annotation.XmlElement;
-import javax.xml.bind.annotation.XmlRootElement;
+import java.util.Objects;
+
+import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
+import com.fasterxml.jackson.annotation.JsonInclude;
+import com.fasterxml.jackson.annotation.JsonInclude.Include;
+import com.netscape.certsrv.util.JSONSerializer;
 
 /**
  * @author alee
  *
  */
-@XmlRootElement(name="AdminSetupResponse")
-@XmlAccessorType(XmlAccessType.FIELD)
-public class AdminSetupResponse {
+@JsonInclude(Include.NON_NULL)
+@JsonIgnoreProperties(ignoreUnknown=true)
+public class AdminSetupResponse implements JSONSerializer {
 
-    @XmlElement
     protected SystemCertData adminCert = new SystemCertData();
 
     public SystemCertData getAdminCert() {
@@ -40,4 +41,22 @@ public class AdminSetupResponse {
     public void setAdminCert(SystemCertData adminCert) {
         this.adminCert = adminCert;
     }
+
+    @Override
+    public int hashCode() {
+        return Objects.hash(adminCert);
+    }
+
+    @Override
+    public boolean equals(Object obj) {
+        if (this == obj)
+            return true;
+        if (obj == null)
+            return false;
+        if (getClass() != obj.getClass())
+            return false;
+        AdminSetupResponse other = (AdminSetupResponse) obj;
+        return Objects.equals(adminCert, other.adminCert);
+    }
+
 }
