@@ -17,20 +17,21 @@
 // --- END COPYRIGHT BLOCK ---
 package com.netscape.certsrv.system;
 
-import javax.xml.bind.annotation.XmlAccessType;
-import javax.xml.bind.annotation.XmlAccessorType;
-import javax.xml.bind.annotation.XmlElement;
-import javax.xml.bind.annotation.XmlRootElement;
+import java.util.Objects;
+
+import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
+import com.fasterxml.jackson.annotation.JsonInclude;
+import com.fasterxml.jackson.annotation.JsonInclude.Include;
+import com.netscape.certsrv.util.JSONSerializer;
 
 /**
  * @author alee
  *
  */
-@XmlRootElement(name="InstallToken")
-@XmlAccessorType(XmlAccessType.FIELD)
-public class InstallToken {
+@JsonInclude(Include.NON_NULL)
+@JsonIgnoreProperties(ignoreUnknown=true)
+public class InstallToken implements JSONSerializer {
 
-    @XmlElement
     private String token;
 
     public InstallToken(String token) {
@@ -38,7 +39,6 @@ public class InstallToken {
     }
 
     public InstallToken() {
-        // required by jaxb
     }
 
     public String getToken() {
@@ -48,4 +48,22 @@ public class InstallToken {
     public void setToken(String token) {
         this.token = token;
     }
+
+    @Override
+    public int hashCode() {
+        return Objects.hash(token);
+    }
+
+    @Override
+    public boolean equals(Object obj) {
+        if (this == obj)
+            return true;
+        if (obj == null)
+            return false;
+        if (getClass() != obj.getClass())
+            return false;
+        InstallToken other = (InstallToken) obj;
+        return Objects.equals(token, other.token);
+    }
+
 }
