@@ -20,18 +20,19 @@ package com.netscape.certsrv.tps.token;
 
 import java.util.Collection;
 
-import javax.xml.bind.annotation.XmlElementRef;
-import javax.xml.bind.annotation.XmlRootElement;
-
+import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
+import com.fasterxml.jackson.annotation.JsonInclude;
+import com.fasterxml.jackson.annotation.JsonInclude.Include;
 import com.netscape.certsrv.base.DataCollection;
 
 /**
  * @author Endi S. Dewata
  */
-@XmlRootElement(name="Tokens")
+@JsonInclude(Include.NON_NULL)
+@JsonIgnoreProperties(ignoreUnknown=true)
 public class TokenCollection extends DataCollection<TokenData> {
 
-    @XmlElementRef
+    @Override
     public Collection<TokenData> getEntries() {
         return super.getEntries();
     }
