@@ -40,6 +40,7 @@ import java.security.SecureRandom;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
 import java.util.Collection;
+import java.util.Comparator;
 import java.util.Enumeration;
 import java.util.List;
 import java.util.Set;
@@ -53,6 +54,7 @@ import org.mozilla.jss.CryptoManager;
 import org.mozilla.jss.netscape.security.extensions.AccessDescription;
 import org.mozilla.jss.netscape.security.extensions.AuthInfoAccessExtension;
 import org.mozilla.jss.netscape.security.extensions.ExtendedKeyUsageExtension;
+import org.mozilla.jss.netscape.security.extensions.OCSPNoCheckExtension;
 import org.mozilla.jss.netscape.security.pkcs.PKCS10;
 import org.mozilla.jss.netscape.security.util.Cert;
 import org.mozilla.jss.netscape.security.util.DerOutputStream;
@@ -62,11 +64,11 @@ import org.mozilla.jss.netscape.security.util.Utils;
 import org.mozilla.jss.netscape.security.x509.AuthorityKeyIdentifierExtension;
 import org.mozilla.jss.netscape.security.x509.BasicConstraintsExtension;
 import org.mozilla.jss.netscape.security.x509.CPSuri;
-import org.mozilla.jss.netscape.security.x509.CertificateExtensions;
 import org.mozilla.jss.netscape.security.x509.CertificatePoliciesExtension;
 import org.mozilla.jss.netscape.security.x509.CertificatePolicyId;
 import org.mozilla.jss.netscape.security.x509.CertificatePolicyInfo;
 import org.mozilla.jss.netscape.security.x509.Extension;
+import org.mozilla.jss.netscape.security.x509.Extensions;
 import org.mozilla.jss.netscape.security.x509.GeneralName;
 import org.mozilla.jss.netscape.security.x509.GeneralNameInterface;
 import org.mozilla.jss.netscape.security.x509.KeyIdentifier;
@@ -89,6 +91,9 @@ public class NSSDatabase {
     FileAttribute<Set<PosixFilePermission>> FILE_PERMISSIONS =
             PosixFilePermissions.asFileAttribute(PosixFilePermissions.fromString("rw-------"));
 
+    FileAttribute<Set<PosixFilePermission>> DIR_PERMISSIONS =
+            PosixFilePermissions.asFileAttribute(PosixFilePermissions.fromString("rwx------"));
+
     Path path;
     IPasswordStore passwordStore;
 
@@ -143,7 +148,7 @@ public class NSSDatabase {
 
     public void create(String password, boolean enableTrustPolicy) throws Exception {
 
-        logger.info("Creating NSS database in " + path);
+        logger.info("NSSDatabase: Creating NSS database in " + path);
 
         Files.createDirectories(path);
 
@@ -191,7 +196,7 @@ public class NSSDatabase {
 
     public boolean moduleExists(String name) throws Exception {
 
-        logger.info("Checking module " + name);
+        logger.info("NSSDatabase: Checking module " + name);
 
         List<String> command = new ArrayList<>();
         command.add("modutil");
@@ -230,7 +235,7 @@ public class NSSDatabase {
 
     public void addModule(String name, String library) throws Exception {
 
-        logger.info("Installing " + name + " module with " + library);
+        logger.info("NSSDatabase: Installing " + name + " module with " + library);
 
         List<String> command = new ArrayList<>();
         command.add("modutil");
@@ -300,6 +305,15 @@ public class NSSDatabase {
             X509Certificate cert,
             String trustAttributes) throws Exception {
 
+        addCertificate(null, nickname, cert, trustAttributes);
+    }
+
+    public void addCertificate(
+            String tokenName,
+            String nickname,
+            X509Certificate cert,
+            String trustAttributes) throws Exception {
+
         byte[] bytes = CertUtil.toPEM(cert).getBytes();
         Path certPath = null;
 
@@ -307,7 +321,7 @@ public class NSSDatabase {
             certPath = Files.createTempFile("nss-cert-", ".crt", FILE_PERMISSIONS);
             Files.write(certPath, bytes);
 
-            addPEMCertificate(nickname, certPath.toString(), trustAttributes);
+            addPEMCertificate(tokenName, nickname, certPath.toString(), trustAttributes);
 
         } finally {
             if (certPath != null) Files.delete(certPath);
@@ -319,7 +333,21 @@ public class NSSDatabase {
             String filename,
             String trustAttributes) throws Exception {
 
+        addPEMCertificate(
+                null,
+                nickname,
+                filename,
+                trustAttributes);
+    }
+
+    public void addPEMCertificate(
+            String tokenName,
+            String nickname,
+            String filename,
+            String trustAttributes) throws Exception {
+
         Path passwordPath = null;
+        if (trustAttributes == null) trustAttributes = ",,";
 
         try {
             List<String> cmd = new ArrayList<>();
@@ -328,17 +356,25 @@ public class NSSDatabase {
             cmd.add("-d");
             cmd.add(path.toString());
 
-            // TODO: Add support for HSM.
-            String password = passwordStore.getPassword("internal", 0);
+            if (tokenName != null) {
+                cmd.add("-h");
+                cmd.add(tokenName);
+            }
 
-            if (password != null) {
-                passwordPath = Files.createTempFile("nss-password-", ".txt", FILE_PERMISSIONS);
-                logger.info("Storing password into " + passwordPath);
+            if (passwordStore != null) {
 
-                Files.write(passwordPath, password.getBytes());
+                String tag = tokenName == null ? "internal" : "hardware-" + tokenName;
+                String password = passwordStore.getPassword(tag, 0);
 
-                cmd.add("-f");
-                cmd.add(passwordPath.toString());
+                if (password != null) {
+                    passwordPath = Files.createTempFile("nss-password-", ".txt", FILE_PERMISSIONS);
+                    logger.info("NSSDatabase: Storing password into " + passwordPath);
+
+                    Files.write(passwordPath, password.getBytes());
+
+                    cmd.add("-f");
+                    cmd.add(passwordPath.toString());
+                }
             }
 
             // accept PEM or PKCS #7 certificate
@@ -384,13 +420,13 @@ public class NSSDatabase {
             PrintWriter stdin,
             BasicConstraintsExtension extension) throws Exception {
 
-        logger.info("Adding basic constraints extension:");
+        logger.info("NSSDatabase: Adding basic constraints extension:");
 
         cmd.add("-2");
 
         // Is this a CA certificate [y/N]?
         boolean ca = (boolean) extension.get(BasicConstraintsExtension.IS_CA);
-        logger.info("- CA: " + ca);
+        logger.info("NSSDatabase: - CA: " + ca);
         if (ca) {
             stdin.print("y");
         }
@@ -398,13 +434,13 @@ public class NSSDatabase {
 
         // Enter the path length constraint, enter to skip [<0 for unlimited path]: >
         int pathLength = (int) extension.get(BasicConstraintsExtension.PATH_LEN);
-        logger.info("- path length: " + pathLength);
+        logger.info("NSSDatabase: - path length: " + pathLength);
         stdin.print(pathLength);
         stdin.println();
 
         // Is this a critical extension [y/N]?
         if (extension.isCritical()) {
-            logger.info("- critical");
+            logger.info("NSSDatabase: - critical");
             stdin.print("y");
         }
         stdin.println();
@@ -423,7 +459,7 @@ public class NSSDatabase {
             PrintWriter stdin,
             AuthorityKeyIdentifierExtension extension) throws Exception {
 
-        logger.info("Adding AKID extension:");
+        logger.info("NSSDatabase: Adding AKID extension:");
 
         cmd.add("-3");
 
@@ -432,7 +468,7 @@ public class NSSDatabase {
 
         KeyIdentifier keyID = (KeyIdentifier) extension.get(AuthorityKeyIdentifierExtension.KEY_ID);
         String akid = "0x" + Utils.HexEncode(keyID.getIdentifier());
-        logger.info("- AKID: " + akid);
+        logger.info("NSSDatabase: - AKID: " + akid);
 
         // Enter value for the key identifier fields,enter to omit:
         stdin.println(akid);
@@ -463,13 +499,13 @@ public class NSSDatabase {
             PrintWriter stdin,
             SubjectKeyIdentifierExtension extension) throws Exception {
 
-        logger.info("Adding SKID extension:");
+        logger.info("NSSDatabase: Adding SKID extension:");
 
         cmd.add("--extSKID");
 
         KeyIdentifier keyID = (KeyIdentifier) extension.get(SubjectKeyIdentifierExtension.KEY_ID);
         String skid = "0x" + Utils.HexEncode(keyID.getIdentifier());
-        logger.info("- SKID: " + skid);
+        logger.info("NSSDatabase: - SKID: " + skid);
 
         // Enter value for the key identifier fields,enter to omit:
         stdin.println(skid);
@@ -494,7 +530,7 @@ public class NSSDatabase {
             PrintWriter stdin,
             AuthInfoAccessExtension extension) throws Exception {
 
-        logger.info("Adding AIA extension:");
+        logger.info("NSSDatabase: Adding AIA extension:");
 
         cmd.add("--extAIA");
 
@@ -504,13 +540,13 @@ public class NSSDatabase {
 
             ObjectIdentifier method = ad.getMethod();
             if (AuthInfoAccessExtension.METHOD_CA_ISSUERS.equals(method)) {
-                logger.info("- CA issuers");
+                logger.info("NSSDatabase: - CA issuers");
 
                 // Enter access method type for Authority Information Access extension:
                 stdin.println("1");
 
             } else if (AuthInfoAccessExtension.METHOD_OCSP.equals(method)) {
-                logger.info("- OCSP");
+                logger.info("NSSDatabase: - OCSP");
 
                 // Enter access method type for Authority Information Access extension:
                 stdin.println("2");
@@ -535,7 +571,7 @@ public class NSSDatabase {
                 DerValue derValue = new DerValue(new ByteArrayInputStream(bytes));
                 derValue.resetTag(DerValue.tag_IA5String);
                 String uri = derValue.getIA5String();
-                logger.info("  - URI: " + uri);
+                logger.info("NSSDatabase:   - URI: " + uri);
 
                 // Select one of the following general name type:
                 stdin.println("7");
@@ -575,56 +611,56 @@ public class NSSDatabase {
             List<String> cmd,
             KeyUsageExtension extension) throws Exception {
 
-        logger.info("Adding key usage extension:");
+        logger.info("NSSDatabase: Adding key usage extension:");
 
         cmd.add("--keyUsage");
 
         List<String> options = new ArrayList<>();
 
         if (extension.isCritical()) {
-            logger.info("- critical");
+            logger.info("NSSDatabase: - critical");
             options.add("critical");
         }
 
         Boolean digitalSignature = (Boolean) extension.get(KeyUsageExtension.DIGITAL_SIGNATURE);
         if (digitalSignature) {
-            logger.info("- digitalSignature");
+            logger.info("NSSDatabase: - digitalSignature");
             options.add("digitalSignature");
         }
 
         Boolean nonRepudiation = (Boolean) extension.get(KeyUsageExtension.NON_REPUDIATION);
         if (nonRepudiation) {
-            logger.info("- nonRepudiation");
+            logger.info("NSSDatabase: - nonRepudiation");
             options.add("nonRepudiation");
         }
 
         Boolean keyEncipherment = (Boolean) extension.get(KeyUsageExtension.KEY_ENCIPHERMENT);
         if (keyEncipherment) {
-            logger.info("- keyEncipherment");
+            logger.info("NSSDatabase: - keyEncipherment");
             options.add("keyEncipherment");
         }
 
         Boolean dataEncipherment = (Boolean) extension.get(KeyUsageExtension.DATA_ENCIPHERMENT);
         if (dataEncipherment) {
-            logger.info("- dataEncipherment");
+            logger.info("NSSDatabase: - dataEncipherment");
             options.add("dataEncipherment");
         }
 
         Boolean keyAgreement = (Boolean) extension.get(KeyUsageExtension.KEY_AGREEMENT);
         if (keyAgreement) {
-            logger.info("- keyAgreement");
+            logger.info("NSSDatabase: - keyAgreement");
             options.add("keyAgreement");
         }
 
         Boolean certSigning = (Boolean) extension.get(KeyUsageExtension.KEY_CERTSIGN);
         if (certSigning) {
-            logger.info("- certSigning");
+            logger.info("NSSDatabase: - certSigning");
             options.add("certSigning");
         }
 
         Boolean crlSigning = (Boolean) extension.get(KeyUsageExtension.CRL_SIGN);
         if (crlSigning) {
-            logger.info("- crlSigning");
+            logger.info("NSSDatabase: - crlSigning");
             options.add("crlSigning");
         }
 
@@ -644,14 +680,14 @@ public class NSSDatabase {
             List<String> cmd,
             ExtendedKeyUsageExtension extension) throws Exception {
 
-        logger.info("Adding extended key usage extension:");
+        logger.info("NSSDatabase: Adding extended key usage extension:");
 
         cmd.add("--extKeyUsage");
 
         List<String> options = new ArrayList<>();
 
         if (extension.isCritical()) {
-            logger.info("- critical");
+            logger.info("NSSDatabase: - critical");
             options.add("critical");
         }
 
@@ -660,13 +696,21 @@ public class NSSDatabase {
             ObjectIdentifier oid = e.nextElement();
 
             if (ObjectIdentifier.getObjectIdentifier("1.3.6.1.5.5.7.3.1").equals(oid)) {
-                logger.info("- serverAuth");
+                logger.info("NSSDatabase: - serverAuth");
                 options.add("serverAuth");
 
             } else if (ObjectIdentifier.getObjectIdentifier("1.3.6.1.5.5.7.3.2").equals(oid)) {
-                logger.info("- clientAuth");
+                logger.info("NSSDatabase: - clientAuth");
                 options.add("clientAuth");
 
+            } else if (ObjectIdentifier.getObjectIdentifier("1.3.6.1.5.5.7.3.4").equals(oid)) {
+                logger.info("NSSDatabase: - emailProtection");
+                options.add("emailProtection");
+
+            } else if (ObjectIdentifier.getObjectIdentifier("1.3.6.1.5.5.7.3.9").equals(oid)) {
+                logger.info("NSSDatabase: - OCSPSigning");
+                options.add("ocspResponder");
+
             } else {
                 throw new Exception("Unsupported extended key usage: " + oid);
             }
@@ -690,7 +734,7 @@ public class NSSDatabase {
             PrintWriter stdin,
             CertificatePoliciesExtension extension) throws Exception {
 
-        logger.info("Adding certificate policies extension:");
+        logger.info("NSSDatabase: Adding certificate policies extension:");
 
         cmd.add("--extCP");
 
@@ -702,7 +746,7 @@ public class NSSDatabase {
 
             CertificatePolicyId policyID = info.getPolicyIdentifier();
             ObjectIdentifier policyOID = policyID.getIdentifier();
-            logger.info("- " + policyOID);
+            logger.info("NSSDatabase: - " + policyOID);
 
             // Enter a CertPolicy Object Identifier (dotted decimal format)
             // or "any" for AnyPolicy: >
@@ -725,7 +769,7 @@ public class NSSDatabase {
 
                         CPSuri cpsURI = (CPSuri) qualifierInfo.getQualifier();
                         String uri = cpsURI.getURI();
-                        logger.info("  - CPS: " + uri);
+                        logger.info("NSSDatabase:   - CPS: " + uri);
 
                         // Choose the type of qualifier for policy:
                         stdin.println("1");
@@ -759,10 +803,45 @@ public class NSSDatabase {
         stdin.println();
     }
 
+    /**
+     * This method provides the arguments and the standard input for certutil
+     * to create a cert/CSR with OCSP No Check extension.
+     *
+     * @param cmd certutil command and arguments
+     * @param stdin certutil's standard input
+     * @param extension The extension to add
+     * @param tmpDir Temporary directory to store extension value
+     */
+    public void addOCSPNoCheckExtension(
+            List<String> cmd,
+            PrintWriter stdin,
+            OCSPNoCheckExtension extension,
+            Path tmpDir) throws Exception {
+
+        logger.info("NSSDatabase: Adding OCSP No Check extension:");
+
+        cmd.add("--extGeneric");
+
+        ObjectIdentifier oid = extension.getExtensionId();
+        logger.info("NSSDatabase: - OID: " + oid);
+
+        boolean critical = extension.isCritical();
+        logger.info("NSSDatabase: - critical: " + critical);
+        String flag = critical ? "critical" : "not-critical";
+
+        byte[] value = extension.getExtensionValue();
+        logger.info("NSSDatabase: - value: " + (value == null ? null : Utils.base64encodeSingleLine(value)));
+        Path file = tmpDir.resolve("ocsp-no-check.ext");
+        Files.write(file, value);
+
+        cmd.add(oid + ":" + flag + ":" + file);
+    }
+
     public void addExtensions(
             List<String> cmd,
             StringWriter sw,
-            CertificateExtensions extensions) throws Exception {
+            Extensions extensions,
+            Path tmpDir) throws Exception {
 
         PrintWriter stdin = new PrintWriter(sw, true);
 
@@ -795,6 +874,10 @@ public class NSSDatabase {
             } else if (extension instanceof CertificatePoliciesExtension) {
                 CertificatePoliciesExtension certificatePoliciesExtension = (CertificatePoliciesExtension) extension;
                 addCertificatePoliciesExtension(cmd, stdin, certificatePoliciesExtension);
+
+            } else if (extension instanceof OCSPNoCheckExtension) {
+                OCSPNoCheckExtension ocspNoCheckExtension = (OCSPNoCheckExtension) extension;
+                addOCSPNoCheckExtension(cmd, stdin, ocspNoCheckExtension, tmpDir);
             }
         }
     }
@@ -806,32 +889,56 @@ public class NSSDatabase {
             String keySize,
             String curve,
             String hash,
-            CertificateExtensions extensions) throws Exception {
+            Extensions extensions) throws Exception {
 
-        logger.info("Creating certificate signing request for " + subject);
+        return createRequest(
+                null,
+                subject,
+                keyID,
+                keyType,
+                keySize,
+                curve,
+                hash,
+                extensions);
+    }
+
+    public PKCS10 createRequest(
+            String tokenName,
+            String subject,
+            String keyID,
+            String keyType,
+            String keySize,
+            String curve,
+            String hash,
+            Extensions extensions) throws Exception {
+
+        logger.info("NSSDatabase: Creating certificate signing request for " + subject);
+
+        if (tokenName != null) {
+            logger.info("NSSDatabase: - token: " + tokenName);
+        }
 
         if (keyID != null) {
-            logger.info("- key ID: " + keyID);
+            logger.info("NSSDatabase: - key ID: " + keyID);
         }
 
         if (keyType != null) {
-            logger.info("- key type: " + keyType);
+            logger.info("NSSDatabase: - key type: " + keyType);
         }
 
         if (keySize != null) {
-            logger.info("- key size: " + keySize);
+            logger.info("NSSDatabase: - key size: " + keySize);
         }
 
         if (curve != null) {
-            logger.info("- curve: " + curve);
+            logger.info("NSSDatabase: - curve: " + curve);
         }
 
-        Path csrPath = null;
-        Path passwordPath = null;
-        Path noisePath = null;
+        Path tmpDir = null;
 
         try {
-            csrPath = Files.createTempFile("nss-request-", ".csr", FILE_PERMISSIONS);
+            tmpDir = Files.createTempDirectory("pki-nss-", DIR_PERMISSIONS);
+            Path csrPath = tmpDir.resolve("request.der");
 
             // TODO: Use JSS to generate the request.
 
@@ -841,14 +948,19 @@ public class NSSDatabase {
             cmd.add("-d");
             cmd.add(path.toString());
 
+            if (tokenName != null) {
+                cmd.add("-h");
+                cmd.add(tokenName);
+            }
+
             if (passwordStore != null) {
 
-                // TODO: Add support for HSM.
-                String password = passwordStore.getPassword("internal", 0);
+                String tag = tokenName == null ? "internal" : "hardware-" + tokenName;
+                String password = passwordStore.getPassword(tag, 0);
 
                 if (password != null) {
-                    passwordPath = Files.createTempFile("nss-password-", ".txt", FILE_PERMISSIONS);
-                    logger.info("Storing password into " + passwordPath);
+                    Path passwordPath = tmpDir.resolve("password.txt");
+                    logger.info("NSSDatabase: Storing password into " + passwordPath);
 
                     Files.write(passwordPath, password.getBytes());
 
@@ -886,8 +998,8 @@ public class NSSDatabase {
                     cmd.add(curve);
                 }
 
-                noisePath = Files.createTempFile("nss-noise-", ".bin", FILE_PERMISSIONS);
-                logger.info("Storing noise into " + noisePath);
+                Path noisePath = tmpDir.resolve("noise.bin");
+                logger.info("NSSDatabase: Storing noise into " + noisePath);
 
                 byte[] bytes = new byte[2048];
                 SecureRandom random = SecureRandom.getInstance("pkcs11prng", "Mozilla-JSS");
@@ -906,7 +1018,7 @@ public class NSSDatabase {
 
             StringWriter stdin = new StringWriter();
             if (extensions != null) {
-                addExtensions(cmd, stdin, extensions);
+                addExtensions(cmd, stdin, extensions, tmpDir);
             }
 
             debug(cmd);
@@ -925,14 +1037,17 @@ public class NSSDatabase {
                 throw new CLIException("Command failed. RC: " + rc, rc);
             }
 
-            logger.info("Loading CSR from " + csrPath);
+            logger.info("NSSDatabase: Loading CSR from " + csrPath);
             byte[] csrBytes = Files.readAllBytes(csrPath);
             return new PKCS10(csrBytes);
 
         } finally {
-            if (noisePath != null) Files.delete(noisePath);
-            if (passwordPath != null) Files.delete(passwordPath);
-            if (csrPath != null) Files.delete(csrPath);
+            if (tmpDir != null) {
+                Files.walk(tmpDir).
+                    sorted(Comparator.reverseOrder()).
+                    map(Path::toFile).
+                    forEach(File::delete);
+            }
         }
     }
 
@@ -940,17 +1055,53 @@ public class NSSDatabase {
             org.mozilla.jss.crypto.X509Certificate issuer,
             PKCS10 pkcs10,
             Integer monthsValid,
-            CertificateExtensions extensions) throws Exception {
+            String hash,
+            Extensions extensions) throws Exception {
 
-        Path csrPath = null;
-        Path certPath = null;
-        Path passwordPath = null;
+        return createCertificate(
+                issuer,
+                pkcs10,
+                null, // serial number
+                monthsValid,
+                hash,
+                extensions);
+    }
+
+    public X509Certificate createCertificate(
+            org.mozilla.jss.crypto.X509Certificate issuer,
+            PKCS10 pkcs10,
+            String serialNumber,
+            Integer monthsValid,
+            String hash,
+            Extensions extensions) throws Exception {
+
+        return createCertificate(
+                null, // token name
+                issuer,
+                pkcs10,
+                null, // serial number
+                monthsValid,
+                hash,
+                extensions);
+    }
+
+    public X509Certificate createCertificate(
+            String tokenName,
+            org.mozilla.jss.crypto.X509Certificate issuer,
+            PKCS10 pkcs10,
+            String serialNumber,
+            Integer monthsValid,
+            String hash,
+            Extensions extensions) throws Exception {
+
+        Path tmpDir = null;
 
         try {
-            csrPath = Files.createTempFile("nss-request-", ".csr", FILE_PERMISSIONS);
-            certPath = Files.createTempFile("nss-cert-", ".crt", FILE_PERMISSIONS);
+            tmpDir = Files.createTempDirectory("pki-nss-", DIR_PERMISSIONS);
+            Path csrPath = tmpDir.resolve("request.der");
+            Path certPath = tmpDir.resolve("cert.der");
 
-            logger.info("Storing CSR into " + csrPath);
+            logger.info("NSSDatabase: Storing CSR into " + csrPath);
             Files.write(csrPath, pkcs10.toByteArray());
 
             // TODO: Use JSS to issue the certificate.
@@ -961,14 +1112,19 @@ public class NSSDatabase {
             cmd.add("-d");
             cmd.add(path.toString());
 
+            if (tokenName != null) {
+                cmd.add("-h");
+                cmd.add(tokenName);
+            }
+
             if (passwordStore != null) {
 
-                // TODO: Add support for HSM.
-                String password = passwordStore.getPassword("internal", 0);
+                String tag = tokenName == null ? "internal" : "hardware-" + tokenName;
+                String password = passwordStore.getPassword(tag, 0);
 
                 if (password != null) {
-                    passwordPath = Files.createTempFile("nss-password-", ".txt", FILE_PERMISSIONS);
-                    logger.info("Storing password into " + passwordPath);
+                    Path passwordPath = tmpDir.resolve("password.txt");
+                    logger.info("NSSDatabase: Storing password into " + passwordPath);
 
                     Files.write(passwordPath, password.getBytes());
 
@@ -977,6 +1133,11 @@ public class NSSDatabase {
                 }
             }
 
+            if (hash != null) {
+                cmd.add("-Z");
+                cmd.add(hash);
+            }
+
             if (issuer == null) {
                 cmd.add("-x");
 
@@ -991,6 +1152,11 @@ public class NSSDatabase {
             cmd.add("-o");
             cmd.add(certPath.toString());
 
+            if (serialNumber != null) {
+                cmd.add("-m");
+                cmd.add(serialNumber.toString());
+            }
+
             if (monthsValid != null) {
                 cmd.add("-v");
                 cmd.add(monthsValid.toString());
@@ -998,7 +1164,7 @@ public class NSSDatabase {
 
             StringWriter stdin = new StringWriter();
             if (extensions != null) {
-                addExtensions(cmd, stdin, extensions);
+                addExtensions(cmd, stdin, extensions, tmpDir);
             }
 
             debug(cmd);
@@ -1017,14 +1183,17 @@ public class NSSDatabase {
                 throw new CLIException("Command failed. RC: " + rc, rc);
             }
 
-            logger.info("Loading certificate from " + certPath);
+            logger.info("NSSDatabase: Loading certificate from " + certPath);
             byte[] certBytes = Files.readAllBytes(certPath);
             return new X509CertImpl(certBytes);
 
         } finally {
-            if (passwordPath != null) Files.delete(passwordPath);
-            if (certPath != null) Files.delete(certPath);
-            if (csrPath != null) Files.delete(csrPath);
+            if (tmpDir != null) {
+                Files.walk(tmpDir).
+                    sorted(Comparator.reverseOrder()).
+                    map(Path::toFile).
+                    forEach(File::delete);
+            }
         }
     }
 
@@ -1049,12 +1218,13 @@ public class NSSDatabase {
                 if (quote) sb.append('"');
             }
 
-            logger.debug(sb.toString());
+            logger.debug("NSSDatabase: " + sb);
         }
     }
 
     public void readStdout(Process process) {
         new Thread() {
+            @Override
             public void run() {
                 try (InputStream is = process.getInputStream();
                         InputStreamReader isr = new InputStreamReader(is);
@@ -1062,7 +1232,7 @@ public class NSSDatabase {
 
                     String line;
                     while ((line = in.readLine()) != null) {
-                        logger.info(line);
+                        logger.info("NSSDatabase: " + line);
                     }
 
                 } catch (Exception e) {
@@ -1074,6 +1244,7 @@ public class NSSDatabase {
 
     public void readStderr(Process process) {
         new Thread() {
+            @Override
             public void run() {
                 try (InputStream is = process.getErrorStream();
                         InputStreamReader isr = new InputStreamReader(is);
@@ -1081,7 +1252,7 @@ public class NSSDatabase {
 
                     String line;
                     while ((line = in.readLine()) != null) {
-                        logger.warn(line);
+                        logger.warn("NSSDatabase: " + line);
                     }
 
                 } catch (Exception e) {
