@@ -48,6 +48,4 @@ setup(
         'Programming Language :: Python :: 3.6',
     ],
     python_requires='!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*,!=3.4.*',
-    setup_requires=['pytest-runner'],
-    tests_require=['pytest'],
 )
