@@ -89,7 +89,7 @@ class BannerShowCLI(pki.cli.CLI):
                 self.usage()
                 sys.exit(1)
 
-        instance = pki.server.instance.PKIInstance(instance_name)
+        instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
         if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
@@ -169,7 +169,7 @@ class BannerValidateCLI(pki.cli.CLI):
             else:
 
                 # load banner from instance
-                instance = pki.server.instance.PKIInstance(instance_name)
+                instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
                 if not instance.exists():
                     logger.error('Invalid instance %s.', instance_name)
