@@ -163,7 +163,7 @@ class InstanceCertExportCLI(pki.cli.CLI):
             self.print_help()
             sys.exit(1)
 
-        instance = pki.server.instance.PKIInstance(instance_name)
+        instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
         if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
@@ -247,7 +247,7 @@ class InstanceFindCLI(pki.cli.CLI):
             else:
                 print()
 
-            instance = pki.server.instance.PKIInstance(instance_name)
+            instance = pki.server.instance.PKIServerFactory.create(instance_name)
             instance.load()
 
             InstanceCLI.print_instance(instance)
@@ -300,7 +300,7 @@ class InstanceShowCLI(pki.cli.CLI):
 
         instance_name = args[0]
 
-        instance = pki.server.instance.PKIInstance(instance_name)
+        instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
         if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
@@ -358,7 +358,7 @@ class InstanceStartCLI(pki.cli.CLI):
 
         instance_name = args[0]
 
-        instance = pki.server.instance.PKIInstance(instance_name)
+        instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
         if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
@@ -421,7 +421,7 @@ class InstanceStopCLI(pki.cli.CLI):
 
         instance_name = args[0]
 
-        instance = pki.server.instance.PKIInstance(instance_name)
+        instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
         if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
@@ -498,7 +498,7 @@ class InstanceMigrateCLI(pki.cli.CLI):
 
         module = self.get_top_module().find_module('migrate')
 
-        instance = pki.server.instance.PKIInstance(instance_name)
+        instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
         if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
@@ -564,7 +564,7 @@ class InstanceNuxwdogEnableCLI(pki.cli.CLI):
         module = self.get_top_module().find_module('nuxwdog-enable')
         module = pki.server.cli.nuxwdog.NuxwdogEnableCLI()
 
-        instance = pki.server.instance.PKIInstance(instance_name)
+        instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
         if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
@@ -627,7 +627,7 @@ class InstanceNuxwdogDisableCLI(pki.cli.CLI):
 
         module = self.get_top_module().find_module('nuxwdog-disable')
 
-        instance = pki.server.instance.PKIInstance(instance_name)
+        instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
         if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
@@ -721,7 +721,7 @@ class InstanceExternalCertAddCLI(pki.cli.CLI):
             self.print_help()
             sys.exit(1)
 
-        instance = pki.server.instance.PKIInstance(instance_name)
+        instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
         if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
@@ -820,7 +820,7 @@ class InstanceExternalCertDeleteCLI(pki.cli.CLI):
             self.print_help()
             sys.exit(1)
 
-        instance = pki.server.instance.PKIInstance(instance_name)
+        instance = pki.server.instance.PKIServerFactory.create(instance_name)
 
         if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
