@@ -34,6 +34,7 @@ import pki.server.cli.audit
 import pki.server.cli.config
 import pki.server.cli.db
 import pki.server.cli.group
+import pki.server.cli.subsystem
 import pki.server.cli.user
 import pki.server.instance
 
@@ -46,6 +47,8 @@ class TKSCLI(pki.cli.CLI):
         super(TKSCLI, self).__init__(
             'tks', 'TKS management commands')
 
+        self.add_module(pki.server.cli.subsystem.SubsystemDeployCLI(self))
+        self.add_module(pki.server.cli.subsystem.SubsystemUndeployCLI(self))
         self.add_module(pki.server.cli.audit.AuditCLI(self))
         self.add_module(TKSCloneCLI())
         self.add_module(pki.server.cli.config.SubsystemConfigCLI(self))
@@ -142,7 +145,7 @@ class TKSClonePrepareCLI(pki.cli.CLI):
             self.print_help()
             sys.exit(1)
 
-        instance = pki.server.instance.PKIInstance(instance_name)
+        instance = pki.server.instance.PKIServerFactory.create(instance_name)
         if not instance.exists():
             logger.error('Invalid instance %s.', instance_name)
             sys.exit(1)
