@@ -23,11 +23,11 @@ from __future__ import absolute_import
 from __future__ import print_function
 import argparse
 import getpass
+import json
 import ldap
 import logging
 import os
 import string
-import xml.etree.ElementTree as ET
 
 from six.moves import input  # pylint: disable=W0622,F0401
 from six.moves import configparser  # pylint: disable=F0401
@@ -44,7 +44,7 @@ from . import pkiconfig as config
 from . import pkimessages as log
 from . import pkilogging
 
-logger = logging.getLogger('pkiparser')
+logger = logging.getLogger(__name__)
 
 
 class PKIConfigParser:
@@ -548,8 +548,8 @@ class PKIConfigParser:
             verify=False)
         client = pki.system.SystemStatusClient(conn, subsystem=system_type)
         response = client.get_status()
-        root = ET.fromstring(response)
-        return root.findtext("Status")
+        json_response = json.loads(response)
+        return json_response['Response']['Status']
 
     def normalize_cert_token(self, name):
 
@@ -1025,6 +1025,15 @@ class PKIConfigParser:
                 # Stand-alone PKI
                 self.mdict['pki_import_admin_cert'] = 'false'
 
+            if config.str2bool(self.mdict['pki_standalone']) \
+                    or config.str2bool(self.mdict['pki_external']) \
+                    and self.mdict['pki_subsystem'] in ['KRA', 'OCSP']:
+
+                if not config.str2bool(self.mdict['pki_external_step_two']):
+                    self.mdict['pki_import_admin_cert'] = 'False'
+                else:
+                    self.mdict['pki_import_admin_cert'] = 'True'
+
             self.mdict['pki_ca_signing_tag'] = "signing"
             if self.mdict['pki_subsystem'] == "CA":
                 self.mdict['pki_ocsp_signing_tag'] = "ocsp_signing"
