@@ -328,7 +328,7 @@ class PKIInstance(pki.server.PKIServer):
 
         # install PKI libraries in common/lib
         for filename in [
-                'jss4.jar',
+                'jss.jar',
                 'ldapjdk.jar',
                 'pki-cmsutil.jar',
                 'pki-tomcat.jar',
@@ -496,13 +496,7 @@ class PKIInstance(pki.server.PKIServer):
 
     def get_sslserver_cert_nickname(self):
 
-        # Load SSL server cert nickname from server.xml
-
-        server_config = self.get_server_config()
-        connector = server_config.get_connector('Secure')
-        sslhost = server_config.get_sslhost(connector)
-        sslcert = server_config.get_sslcert(sslhost)
-        nickname = sslcert.get('certificateKeyAlias')
+        nickname = super(PKIInstance, self).get_sslserver_cert_nickname()
 
         if nickname:
             return nickname
@@ -516,28 +510,23 @@ class PKIInstance(pki.server.PKIServer):
 
     def set_sslserver_cert_nickname(self, nickname, token=None):
 
+        super(PKIInstance, self).set_sslserver_cert_nickname(nickname, token)
+
         if pki.nssdb.normalize_token(token):
-            nickname = token + ':' + nickname
+            fullname = token + ':' + nickname
+        else:
+            fullname = nickname
 
         # Store SSL server cert nickname into serverCertNick.conf
         # TODO: Remove serverCertNick.conf
 
         server_cert_nick_conf = os.path.join(self.conf_dir, 'serverCertNick.conf')
         with open(server_cert_nick_conf, 'w') as f:
-            f.write(nickname + '\n')
+            f.write(fullname + '\n')
 
         os.chown(server_cert_nick_conf, self.uid, self.gid)
         os.chmod(server_cert_nick_conf, pki.server.DEFAULT_FILE_MODE)
 
-        # Store SSL server cert nickname into server.xml
-
-        server_config = self.get_server_config()
-        connector = server_config.get_connector('Secure')
-        sslhost = server_config.get_sslhost(connector)
-        sslcert = server_config.get_sslcert(sslhost)
-        sslcert.set('certificateKeyAlias', nickname)
-        server_config.save()
-
     def banner_installed(self):
         return os.path.exists(self.banner_file)
 
@@ -545,6 +534,16 @@ class PKIInstance(pki.server.PKIServer):
         with io.open(self.banner_file) as f:
             return f.read().strip()
 
+    def validate_banner(self):
+
+        if not self.banner_installed():
+            return
+
+        banner = self.get_banner()
+
+        if not banner:
+            raise Exception('Banner is empty')
+
     def __repr__(self):
         if self.version == 9:
             return "Dogtag 9 " + self.name
@@ -600,7 +599,7 @@ class PKIInstance(pki.server.PKIServer):
         if cert_id == 'sslserver' or cert_id == 'subsystem':
             # Update for all subsystems
             for subsystem in self.get_subsystems():
-                subsystem.update_subsystem_cert(cert)
+                subsystem.update_system_cert(cert)
                 subsystem.save()
         else:
             # Extract subsystem_name from cert_id
@@ -610,7 +609,7 @@ class PKIInstance(pki.server.PKIServer):
             subsystem = self.get_subsystem(subsystem_name)
 
             if subsystem:
-                subsystem.update_subsystem_cert(cert)
+                subsystem.update_system_cert(cert)
                 subsystem.save()
             else:
                 raise pki.server.PKIServerException(
@@ -837,6 +836,10 @@ class PKIInstance(pki.server.PKIServer):
             nssdb.close()
             shutil.rmtree(tmpdir)
 
+    def init(self):
+        super(PKIInstance, self).init()
+        self.validate_banner()
+
     @classmethod
     def instances(cls):
 
