@@ -36,7 +36,7 @@ from pki.server.deployment.pkiparser import PKIConfigParser
 from pki.server.deployment import pkilogging
 from pki.server.deployment import pkimessages as log
 
-logger = logging.getLogger('pkidestroy')
+logger = logging.getLogger(__name__)
 
 deployer = pki.server.deployment.PKIDeployer()
 
@@ -99,6 +99,12 @@ def main(argv):
         help='remove subsystem logs'
     )
 
+    parser.optional.add_argument(
+        '--log-file',
+        dest='log_file',
+        action='store',
+        help='Log file')
+
     args = parser.process_command_line_arguments()
 
     interactive = False
@@ -223,15 +229,19 @@ def main(argv):
         user=deployer.mdict['pki_user'],
         group=deployer.mdict['pki_group'])
 
-    log_dir = config.PKI_DEPLOYMENT_LOG_ROOT
-    log_name = "pki" + "-" +\
-               deployer.subsystem_name.lower() +\
-               "-" + "destroy" + "." +\
-               deployer.log_timestamp + "." + "log"
-    log_file = os.path.join(log_dir, log_name)
+    if args.log_file:
+        log_file = args.log_file
+    else:
+        log_dir = config.PKI_DEPLOYMENT_LOG_ROOT
+        log_name = "pki" + "-" +\
+                   deployer.subsystem_name.lower() +\
+                   "-" + "destroy" + "." +\
+                   deployer.log_timestamp + "." + "log"
+        log_file = os.path.join(log_dir, log_name)
+
     print('Uninstallation log: %s' % log_file)
 
-    pkilogging.enable_pki_logger(log_file, 'pkidestroy')
+    pkilogging.enable_pki_logger(log_file)
 
     # Add force_destroy to master dictionary
     parser.mdict['pki_force_destroy'] = force_destroy
