@@ -21,17 +21,19 @@
 
 from __future__ import absolute_import
 
+import datetime
 import functools
 import json
 import logging
 import os
 import pwd
 import re
+import requests
 import shutil
 import socket
 import subprocess
 import tempfile
-import xml.etree.ElementTree as ET
+import time
 
 import ldap
 import ldap.filter
@@ -55,10 +57,10 @@ class PKISubsystem(object):
         self.instance = instance
         self.name = subsystem_name  # e.g. ca, kra
 
-        if instance.version >= 10:
-            self.base_dir = os.path.join(self.instance.base_dir, self.name)
-        else:
+        if hasattr(instance, 'version') and instance.version < 10:
             self.base_dir = instance.base_dir
+        else:
+            self.base_dir = os.path.join(self.instance.base_dir, self.name)
 
         self.conf_dir = os.path.join(self.base_dir, 'conf')
         self.cs_conf = os.path.join(self.conf_dir, 'CS.cfg')
@@ -154,6 +156,8 @@ class PKISubsystem(object):
 
     def get_subsystem_cert(self, cert_id):
 
+        logger.debug('get_subsystem_cert')
+
         cert = self.get_cert_info(cert_id)
 
         # If nickname is empty, then cert cannot be queried for in NSSDB
@@ -188,6 +192,7 @@ class PKISubsystem(object):
 
     def get_nssdb_cert_info(self, cert_id):
 
+        logger.info('get_nssdb_cert_info')
         logger.info('Getting %s cert info from NSS database', cert_id)
 
         nickname = self.config.get('%s.%s.nickname' % (self.name, cert_id))
@@ -199,7 +204,7 @@ class PKISubsystem(object):
         finally:
             nssdb.close()
 
-    def update_subsystem_cert(self, cert):
+    def update_system_cert(self, cert):
         cert_id = cert['id']
         self.config['%s.%s.nickname' % (self.name, cert_id)] = cert.get('nickname')
         self.config['%s.%s.tokenname' % (self.name, cert_id)] = cert.get('token')
@@ -217,9 +222,12 @@ class PKISubsystem(object):
 
         logger.debug('Command: %s', ' '.join(cmd))
 
-        subprocess.check_output(
-            cmd,
-            stderr=subprocess.STDOUT)
+        try:
+            print(subprocess.check_output(
+                cmd,
+                stderr=subprocess.STDOUT))
+        except subprocess.CalledProcessError as v:
+            print("pki-server subsystem-cert-validate stdout output:\n", v.output)
 
     def export_system_cert(
             self,
@@ -377,20 +385,66 @@ class PKISubsystem(object):
             protocol=protocol,
             hostname=socket.getfqdn(),
             port=port,
-            accept='application/xml',
+            accept='application/json',
             trust_env=False,
             verify=False)
 
         client = pki.system.SystemStatusClient(connection, subsystem=self.name)
         response = client.get_status(timeout=timeout)
-
-        root = ET.fromstring(response)
-        status = root.findtext('Status')
+        json_response = json.loads(response)
+        status = json_response['Response']['Status']
 
         logger.info('Subsystem status: %s', status)
         return status == 'running'
 
-    def enable(self):
+    def wait_for_startup(self, startup_timeout=None, request_timeout=None):
+        """
+        Wait for subsystem to become ready to serve requests.
+
+        :param startup_timeout: Total timeout. Unsuccessful status requests will
+            be retried until this timeout is exceeded. Default: None.
+        :param request_timeout: Connect/receive timeout for each individual
+            status request. Default: None.
+        """
+
+        fips_mode = pki.FIPS.is_enabled()
+
+        # must use 'http' protocol when FIPS mode is enabled
+        secure_connection = not fips_mode
+
+        start_time = datetime.datetime.today()
+        ready = False
+        counter = 0
+
+        while not ready:
+            try:
+                time.sleep(1)
+
+                ready = self.is_ready(
+                    secure_connection=secure_connection,
+                    timeout=request_timeout)
+
+            except requests.exceptions.SSLError as exc:
+                max_retry_error = exc.args[0]
+                reason = getattr(max_retry_error, 'reason')
+                raise Exception('Server unreachable due to SSL error: %s' % reason) from exc
+
+            except pki.RETRYABLE_EXCEPTIONS as exc:
+
+                stop_time = datetime.datetime.today()
+                counter = (stop_time - start_time).total_seconds()
+
+                if startup_timeout is not None and counter >= startup_timeout:
+                    raise Exception('%s subsystem did not start after %ds' %
+                                    (self.type, startup_timeout)) from exc
+
+                logger.info(
+                    'Waiting for %s subsystem to start (%ds)',
+                    self.type,
+                    int(round(counter)))
+
+    def enable(self, wait=False, max_wait=60, timeout=None):
+
         if os.path.exists(self.doc_base):
             # deploy custom subsystem if exists
             doc_base = self.doc_base
@@ -400,10 +454,25 @@ class PKISubsystem(object):
             # /usr/share/pki/<subsystem>/webapps/<subsystem>
             doc_base = None
 
-        self.instance.deploy_webapp(self.name, self.default_context_xml, doc_base)
+        self.instance.deploy_webapp(
+            self.name,
+            self.default_context_xml,
+            doc_base=doc_base,
+            wait=wait,
+            max_wait=max_wait,
+            timeout=timeout)
 
-    def disable(self):
-        self.instance.undeploy_webapp(self.name)
+    def disable(self, wait=False, max_wait=60, timeout=None):
+
+        self.instance.undeploy_webapp(
+            self.name,
+            wait=wait,
+            max_wait=max_wait,
+            timeout=timeout)
+
+    def restart(self, wait=False, max_wait=60, timeout=None):
+        self.disable(wait=True, max_wait=max_wait, timeout=timeout)
+        self.enable(wait=wait, max_wait=max_wait, timeout=timeout)
 
     def open_database(self, name='internaldb', bind_dn=None,
                       bind_password=None):
@@ -663,7 +732,13 @@ class PKISubsystem(object):
 
     def get_startup_tests(self):
         # Split the line 'selftest.container.selftests.startup'
-        available_tests = self.config['selftests.container.order.startup'].split(',')
+        v = self.config.get('selftests.container.order.startup', '').strip()
+        if len(v) == 0:
+            # special case; empty value -> empty list
+            available_tests = []
+        else:
+            available_tests = v.split(',')
+
         target_tests = {}
         for testInfo in available_tests:
             temp = testInfo.split(':')
@@ -883,8 +958,6 @@ class PKISubsystem(object):
             replication_security=None,
             replication_port=None,
             master_replication_port=None,
-            setup_db_manager=False,
-            setup_vlv_indexes=False,
             as_current_user=False):
 
         cmd = [self.name + '-db-init']
@@ -916,12 +989,6 @@ class PKISubsystem(object):
         if master_replication_port:
             cmd.extend(['--master-replication-port', master_replication_port])
 
-        if setup_db_manager:
-            cmd.append('--setup-db-manager')
-
-        if setup_vlv_indexes:
-            cmd.append('--setup-vlv-indexes')
-
         if logger.isEnabledFor(logging.DEBUG):
             cmd.append('--debug')
 
@@ -960,18 +1027,9 @@ class PKISubsystem(object):
 
         self.run(cmd, as_current_user=as_current_user)
 
-    def request_range(self, master_url, install_token, range_type):
+    def find_vlv(self, as_current_user=False):
 
-        cmd = [
-            'pki',
-            '-d', self.instance.nssdb_dir,
-            '-f', self.instance.password_conf,
-            '-U', master_url,
-            '%s-range-request' % self.name,
-            range_type,
-            '--session', install_token.token,
-            '--output-format', 'json'
-        ]
+        cmd = [self.name + '-db-vlv-find']
 
         if logger.isEnabledFor(logging.DEBUG):
             cmd.append('--debug')
@@ -979,28 +1037,98 @@ class PKISubsystem(object):
         elif logger.isEnabledFor(logging.INFO):
             cmd.append('--verbose')
 
-        logger.debug('Command: %s', ' '.join(cmd))
-        output = subprocess.check_output(cmd)
+        self.run(cmd, as_current_user=as_current_user)
 
-        return json.loads(output.decode())
+    def add_vlv(self, as_current_user=False):
 
-    def request_ranges(self, master_url, install_token):
+        cmd = [self.name + '-db-vlv-add']
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        self.run(cmd, as_current_user=as_current_user)
+
+    def delete_vlv(self, as_current_user=False):
+
+        cmd = [self.name + '-db-vlv-del']
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        self.run(cmd, as_current_user=as_current_user)
+
+    def reindex_vlv(self, as_current_user=False):
+
+        cmd = [self.name + '-db-vlv-reindex']
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        self.run(cmd, as_current_user=as_current_user)
+
+    def request_range(self, master_url, range_type, session_id=None, install_token=None):
+
+        tmpdir = tempfile.mkdtemp()
+        try:
+            if not install_token:
+                install_token = os.path.join(tmpdir, 'install-token')
+                with open(install_token, 'w') as f:
+                    f.write(session_id)
+
+            cmd = [
+                'pki',
+                '-d', self.instance.nssdb_dir,
+                '-f', self.instance.password_conf,
+                '-U', master_url,
+                '%s-range-request' % self.name,
+                range_type,
+                '--install-token', install_token,
+                '--output-format', 'json'
+            ]
+
+            if logger.isEnabledFor(logging.DEBUG):
+                cmd.append('--debug')
+
+            elif logger.isEnabledFor(logging.INFO):
+                cmd.append('--verbose')
+
+            logger.debug('Command: %s', ' '.join(cmd))
+            output = subprocess.check_output(cmd)
+
+            return json.loads(output.decode())
+
+        finally:
+            shutil.rmtree(tmpdir)
+
+    def request_ranges(self, master_url, session_id=None, install_token=None):
 
         logger.info('Requesting request ID range')
 
-        request_range = self.request_range(master_url, install_token, 'request')
+        request_range = self.request_range(
+            master_url, 'request', session_id=session_id, install_token=install_token)
         self.config['dbs.beginRequestNumber'] = request_range['begin']
         self.config['dbs.endRequestNumber'] = request_range['end']
 
         logger.info('Requesting serial number range')
 
-        serial_range = self.request_range(master_url, install_token, 'serialNo')
+        serial_range = self.request_range(
+            master_url, 'serialNo', session_id=session_id, install_token=install_token)
         self.config['dbs.beginSerialNumber'] = serial_range['begin']
         self.config['dbs.endSerialNumber'] = serial_range['end']
 
         logger.info('Requesting replica ID range')
 
-        replica_range = self.request_range(master_url, install_token, 'replicaId')
+        replica_range = self.request_range(
+            master_url, 'replicaId', session_id=session_id, install_token=install_token)
         self.config['dbs.beginReplicaNumber'] = replica_range['begin']
         self.config['dbs.endReplicaNumber'] = replica_range['end']
 
@@ -1020,32 +1148,42 @@ class PKISubsystem(object):
 
         self.run(cmd, as_current_user=as_current_user)
 
-    def retrieve_config(self, master_url, names, substores, install_token):
+    def retrieve_config(self, master_url, names, substores, session_id=None, install_token=None):
 
-        cmd = [
-            'pki',
-            '-d', self.instance.nssdb_dir,
-            '-f', self.instance.password_conf,
-            '-U', master_url,
-            '%s-config-export' % self.name,
-            '--names', ','.join(names),
-            '--substores', ','.join(substores),
-            '--session', install_token.token,
-            '--output-format', 'json'
-        ]
+        tmpdir = tempfile.mkdtemp()
+        try:
+            if not install_token:
+                install_token = os.path.join(tmpdir, 'install-token')
+                with open(install_token, 'w') as f:
+                    f.write(session_id)
 
-        if logger.isEnabledFor(logging.DEBUG):
-            cmd.append('--debug')
+            cmd = [
+                'pki',
+                '-d', self.instance.nssdb_dir,
+                '-f', self.instance.password_conf,
+                '-U', master_url,
+                '%s-config-export' % self.name,
+                '--names', ','.join(names),
+                '--substores', ','.join(substores),
+                '--install-token', install_token,
+                '--output-format', 'json'
+            ]
 
-        elif logger.isEnabledFor(logging.INFO):
-            cmd.append('--verbose')
+            if logger.isEnabledFor(logging.DEBUG):
+                cmd.append('--debug')
 
-        logger.debug('Command: %s', ' '.join(cmd))
-        output = subprocess.check_output(cmd)
+            elif logger.isEnabledFor(logging.INFO):
+                cmd.append('--verbose')
 
-        return json.loads(output.decode())
+            logger.debug('Command: %s', ' '.join(cmd))
+            output = subprocess.check_output(cmd)
 
-    def update_config(self, master_url, install_token):
+            return json.loads(output.decode())
+
+        finally:
+            shutil.rmtree(tmpdir)
+
+    def update_config(self, master_url, session_id=None, install_token=None):
 
         logger.info('Updating configuration')
 
@@ -1071,7 +1209,8 @@ class PKISubsystem(object):
         else:
             names.append('cloning.ca.type')
 
-        config = self.retrieve_config(master_url, names, substores, install_token)
+        config = self.retrieve_config(
+            master_url, names, substores, session_id=session_id, install_token=install_token)
         properties = config['properties']
 
         for name in properties:
@@ -1105,9 +1244,25 @@ class PKISubsystem(object):
                 master_port == replica_port:
             raise Exception('Master and replica must not share LDAP database')
 
+    def configure_security_domain(
+            self,
+            sd_type,
+            name,
+            hostname,
+            port,
+            secure_port):
+
+        self.config['securitydomain.select'] = sd_type
+        self.config['securitydomain.name'] = name
+        self.config['securitydomain.host'] = hostname
+        self.config['securitydomain.httpport'] = port
+        self.config['securitydomain.httpsadminport'] = secure_port
+        self.config['securitydomain.httpsagentport'] = secure_port
+        self.config['securitydomain.httpseeport'] = secure_port
+
     def create_security_domain(self, as_current_user=False):
 
-        cmd = ['sd-create']
+        cmd = [self.name + '-sd-create']
 
         if logger.isEnabledFor(logging.DEBUG):
             cmd.append('--debug')
@@ -1128,7 +1283,7 @@ class PKISubsystem(object):
             as_current_user=False):
 
         cmd = [
-            'sd-host-add',
+            self.name + '-sd-host-add',
             '--hostname', hostname
         ]
 
@@ -1158,47 +1313,55 @@ class PKISubsystem(object):
 
     def join_security_domain(
             self,
-            install_token,
+            sd_url,
             host_id,
             hostname,
             unsecure_port='8080',
             secure_port='8443',
             domain_manager=False,
-            clone=False):
+            clone=False,
+            session_id=None,
+            install_token=None):
 
-        sd_hostname = self.config['securitydomain.host']
-        sd_port = self.config['securitydomain.httpsadminport']
-        sd_url = 'https://%s:%s' % (sd_hostname, sd_port)
+        tmpdir = tempfile.mkdtemp()
+        try:
+            if not install_token:
+                install_token = os.path.join(tmpdir, 'install-token')
+                with open(install_token, 'w') as f:
+                    f.write(session_id)
 
-        cmd = [
-            'pki',
-            '-d', self.instance.nssdb_dir,
-            '-f', self.instance.password_conf,
-            '-U', sd_url,
-            'securitydomain-join',
-            '--session', install_token.token,
-            '--type', self.type,
-            '--hostname', hostname,
-            '--unsecure-port', unsecure_port,
-            '--secure-port', secure_port
-        ]
+            cmd = [
+                'pki',
+                '-d', self.instance.nssdb_dir,
+                '-f', self.instance.password_conf,
+                '-U', sd_url,
+                'securitydomain-join',
+                '--install-token', install_token,
+                '--type', self.type,
+                '--hostname', hostname,
+                '--unsecure-port', unsecure_port,
+                '--secure-port', secure_port
+            ]
 
-        if domain_manager:
-            cmd.append('--domain-manager')
+            if domain_manager:
+                cmd.append('--domain-manager')
 
-        if clone:
-            cmd.append('--clone')
+            if clone:
+                cmd.append('--clone')
 
-        if logger.isEnabledFor(logging.DEBUG):
-            cmd.append('--debug')
+            if logger.isEnabledFor(logging.DEBUG):
+                cmd.append('--debug')
 
-        elif logger.isEnabledFor(logging.INFO):
-            cmd.append('--verbose')
+            elif logger.isEnabledFor(logging.INFO):
+                cmd.append('--verbose')
 
-        cmd.append(host_id)
+            cmd.append(host_id)
 
-        logger.debug('Command: %s', ' '.join(cmd))
-        subprocess.check_call(cmd)
+            logger.debug('Command: %s', ' '.join(cmd))
+            subprocess.check_call(cmd)
+
+        finally:
+            shutil.rmtree(tmpdir)
 
     def find_groups(self, as_current_user=False):
 
@@ -1300,54 +1463,66 @@ class PKISubsystem(object):
                  full_name=None,
                  email=None,
                  password=None,
+                 password_file=None,
                  phone=None,
                  user_type=None,
                  state=None,
                  tps_profiles=None,
                  as_current_user=False):
 
-        cmd = [self.name + '-user-add']
+        tmpdir = tempfile.mkdtemp()
 
-        if full_name:
-            cmd.append('--full-name')
-            cmd.append(full_name)
+        try:
+            if password and not password_file:
+                password_file = os.path.join(tmpdir, 'password.txt')
+                with open(password_file, 'w') as f:
+                    f.write(password)
 
-        if email:
-            cmd.append('--email')
-            cmd.append(email)
+            cmd = [self.name + '-user-add']
 
-        if password:
-            cmd.append('--password')
-            cmd.append(password)
+            if full_name:
+                cmd.append('--full-name')
+                cmd.append(full_name)
 
-        if phone:
-            cmd.append('--phone')
-            cmd.append(phone)
+            if email:
+                cmd.append('--email')
+                cmd.append(email)
 
-        if user_type:
-            cmd.append('--type')
-            cmd.append(user_type)
+            if password_file:
+                cmd.append('--password-file')
+                cmd.append(password_file)
 
-        if state:
-            cmd.append('--state')
-            cmd.append(state)
+            if phone:
+                cmd.append('--phone')
+                cmd.append(phone)
 
-        if tps_profiles:
-            cmd.append('--tps-profiles')
-            cmd.append(','.join(tps_profiles))
+            if user_type:
+                cmd.append('--type')
+                cmd.append(user_type)
 
-        if logger.isEnabledFor(logging.DEBUG):
-            cmd.append('--debug')
+            if state:
+                cmd.append('--state')
+                cmd.append(state)
 
-        elif logger.isEnabledFor(logging.INFO):
-            cmd.append('--verbose')
+            if tps_profiles:
+                cmd.append('--tps-profiles')
+                cmd.append(','.join(tps_profiles))
 
-        cmd.append(user_id)
+            if logger.isEnabledFor(logging.DEBUG):
+                cmd.append('--debug')
 
-        self.run(
-            cmd,
-            as_current_user=as_current_user,
-            capture_output=True)
+            elif logger.isEnabledFor(logging.INFO):
+                cmd.append('--verbose')
+
+            cmd.append(user_id)
+
+            self.run(
+                cmd,
+                as_current_user=as_current_user,
+                capture_output=True)
+
+        finally:
+            shutil.rmtree(tmpdir)
 
     def modify_user(self, user_id, add_see_also=None, del_see_also=None,
                     as_current_user=False):
@@ -1516,6 +1691,18 @@ class CASubsystem(PKISubsystem):
 
         self.run(cmd, as_current_user=as_current_user)
 
+    def find_certs(self, as_current_user=False):
+
+        cmd = ['ca-cert-find']
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        self.run(cmd, as_current_user=as_current_user)
+
     def remove_cert(self, serial_number, as_current_user=False):
 
         cmd = ['ca-cert-del']
@@ -1551,11 +1738,11 @@ class CASubsystem(PKISubsystem):
 
         con.close()
 
-        requests = []
+        cert_requests = []
         for entry in entries:
-            requests.append(self.create_request_object(entry))
+            cert_requests.append(self.create_request_object(entry))
 
-        return requests
+        return cert_requests
 
     def get_cert_requests(self, request_id):
 
@@ -1633,6 +1820,50 @@ class OCSPSubsystem(PKISubsystem):
     def __init__(self, instance):
         super(OCSPSubsystem, self).__init__(instance, 'ocsp')
 
+    def find_crl_issuing_point(
+            self,
+            size=None,
+            as_current_user=False):
+
+        cmd = [self.name + '-crl-issuingpoint-find']
+
+        if size:
+            cmd.extend(['--size', size])
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        self.run(cmd, as_current_user=as_current_user)
+
+    def add_crl_issuing_point(
+            self,
+            cert_chain=None,
+            cert_chain_file=None,
+            cert_format=None,
+            as_current_user=False):
+
+        cmd = [self.name + '-crl-issuingpoint-add']
+
+        if cert_chain_file:
+            cmd.extend(['--cert-chain', cert_chain_file])
+
+        if cert_format:
+            cmd.extend(['--cert-format', cert_format])
+
+        if logger.isEnabledFor(logging.DEBUG):
+            cmd.append('--debug')
+
+        elif logger.isEnabledFor(logging.INFO):
+            cmd.append('--verbose')
+
+        self.run(
+            cmd,
+            input=cert_chain,
+            as_current_user=as_current_user)
+
 
 class TKSSubsystem(PKISubsystem):
 
