@@ -18,8 +18,8 @@
 
 package com.netscape.cmstools.ca;
 
-import java.io.FileReader;
-import java.io.IOException;
+import java.nio.file.Files;
+import java.nio.file.Path;
 import java.util.Collection;
 import java.util.Date;
 
@@ -48,10 +48,12 @@ public class CACertFindCLI extends CommandCLI {
         this.certCLI = certCLI;
     }
 
+    @Override
     public void printHelp() {
         formatter.printHelp(getFullName() + " [OPTIONS...]", options);
     }
 
+    @Override
     public void createOptions() {
         Option option = null;
 
@@ -191,6 +193,7 @@ public class CACertFindCLI extends CommandCLI {
         options.addOption(option);
     }
 
+    @Override
     public void execute(CommandLine cmd) throws Exception {
 
         String[] cmdArgs = cmd.getArgs();
@@ -200,29 +203,11 @@ public class CACertFindCLI extends CommandCLI {
         }
 
         CertSearchRequest searchData = null;
-        String fileName = null;
-
-        if (cmd.hasOption("input")) {
-            fileName = cmd.getOptionValue("input");
-            if (fileName == null || fileName.length() < 1) {
-                throw new Exception("No file name specified.");
-            }
-        }
+        String fileName = cmd.getOptionValue("input");
 
         if (fileName != null) {
-            FileReader reader = null;
-            try {
-                reader = new FileReader(fileName);
-                searchData = CertSearchRequest.valueOf(reader);
-
-            } finally {
-                if (reader != null)
-                    try {
-                        reader.close();
-                    } catch (IOException e) {
-                        e.printStackTrace();
-                    }
-            }
+            String content = Files.readString(Path.of(fileName));
+            searchData = CertSearchRequest.fromXML(content);
 
         } else {
             searchData = new CertSearchRequest();
