@@ -20,6 +20,8 @@
  */
 
 var TPS = {
+    PROFILE_ID_PATTERN: /^[a-zA-Z0-9_]+$/,
+    PROPERTY_NAME_PATTERN: /^[a-zA-Z0-9_\.]+$/,
     getElementName: function (component) {
 
         if (component == "Generals") {
@@ -128,6 +130,15 @@ var PropertiesTable = Table.extend({
             return entry.name;
         });
     },
+    addEntry: function(entry) {
+        var self = this;
+
+        if (!entry.name.match(TPS.PROPERTY_NAME_PATTERN)) {
+            throw "Invalid property name: " + entry.name;
+        }
+
+        PropertiesTable.__super__.addEntry.call(self, entry);
+    },
     remove: function(items) {
         var self = this;
 
@@ -419,6 +430,15 @@ var ConfigEntryPage = EntryPage.extend({
 
         self.entry.properties = self.getProperties();
     },
+    saveEntry: function() {
+        var self = this;
+
+        if (!self.entry.profileID.match(TPS.PROFILE_ID_PATTERN)) {
+            throw "Invalid profile ID: " + self.entry.profileID;
+        }
+
+        ConfigEntryPage.__super__.saveEntry.call(self);
+    },
     setProperties: function(properties) {
         var self = this;
 
