@@ -38,9 +38,7 @@ run_dev_junit_tests() {
     CLASSPATH=$CLASSPATH:/usr/share/java/jackson/jackson-smile.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/jackson/jackson-xc.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/resteasy/jaxrs-api.jar
-    CLASSPATH=$CLASSPATH:/usr/share/java/resteasy/resteasy-atom-provider.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/resteasy/resteasy-client.jar
-    CLASSPATH=$CLASSPATH:/usr/share/java/resteasy/resteasy-jaxb-provider.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/resteasy/resteasy-jaxrs.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/resteasy/resteasy-jaxrs-jandex.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/resteasy/resteasy-jackson-provider.jar
@@ -48,20 +46,16 @@ run_dev_junit_tests() {
     CLASSPATH=$CLASSPATH:/usr/share/java/apache-commons-codec.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/apache-commons-logging.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/commons-codec.jar
-    CLASSPATH=$CLASSPATH:/usr/share/java/commons-httpclient.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/idm-console-base-1.1.7.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/idm-console-mcc.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/idm-console-nmclf.jar
-    CLASSPATH=$CLASSPATH:/usr/share/java/jakarta-commons-httpclient.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/jaxb-api.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/jakarta-activation/jakarta.activation.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/ldapjdk.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/apache-commons-lang3.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/istack-commons-runtime.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/servlet.jar
-    CLASSPATH=$CLASSPATH:/usr/share/java/velocity.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/xerces-j2.jar
-    CLASSPATH=$CLASSPATH:/usr/share/java/xml-commons-apis.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/tomcat/catalina.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/tomcat/tomcat-util.jar
     CLASSPATH=$CLASSPATH:/usr/share/java/commons-io.jar
